//  -----------------------------------------------------------------------
//  Copyright 2002-2009  Alfred Steffens Jr.
//  -----------------------------------------------------------------------
//  -----------------------------------------------------------------------
//    Copying Permission:
//
//    attrk is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    attrk is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with attrk (see file called "COPYING"); if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//    02111-1307  USA
//  ------------------------------------------------------------------------


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <stdarg.h>
#include <slang.h>
#include <defs.h>
#include <logprintf.h>

int g_error_display = ERROR_DISPLAY_STDERR;
static char msg[2048];
extern void SLang_message(char *msg);


//  -------------------------------------------------------------------------
//  Function    --  Error_Printf
//
//  Description
//
//      A wrapper for error messages that is independent of the type of
//      output (GUI as opposed to console or file output)
//
//  Arguments
//
//      fmt
//
//          Format string according to the printf syntax.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void Error_Printf(char *fmt, ...)
{
va_list vl;

    //
    //  vl is a pointer to arguments on the function stack
    //  va_start points "vl" to the first argument that comes after "fmt"
    //
    va_start(vl, fmt);

    v_LogFile_Printf(fmt, vl);		// print to logfile

    switch (g_error_display)
    {
        case ERROR_DISPLAY_SLANG:
            vsprintf(msg, fmt, vl);
            SLang_message(msg);
        break;

        default:
            vfprintf(stderr, fmt, vl);	// standard error
        break;

    } // switch

    va_end(vl);

}
//  function -- Error_Printf
//  -------------------------------------------------------------------------

