//  -----------------------------------------------------------------------
//  Copyright 2002--2009  Alfred Steffens Jr.
//  -----------------------------------------------------------------------
//  -----------------------------------------------------------------------
//    Copying Permission:
//
//    attrk is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    attrk is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with attrk (see file called "COPYING"); if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//    02111-1307  USA
//  ------------------------------------------------------------------------


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#define LOGFILE_HOME
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <stdarg.h>

static FILE *flogfile = NULL;
extern char version_number[64];
int numlogtabs=0;


//  -------------------------------------------------------------------------
//  Function    --  LogFile_Open
//
//  Description
//
//      Open the existing log file for appending.
//
//  Arguments
//
//      Void
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void LogFile_Open(void)
{

    flogfile = fopen("logfile.txt", "a+");
    if (flogfile == NULL) {
    	fprintf(stderr,"ERROR: LogFile_Open: Failure\n");
    	exit(1);
    }

}
//  function -- LogFile_Open
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  LogFile_Close
//
//  Description
//
//      Close the log file.  Not intended as the final close
//      (see LogFile_Exit)
//
//  Arguments
//
//      Void
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void LogFile_Close(void)
{
    if (flogfile)
        fclose(flogfile);
    flogfile = NULL;
}
//  function -- LogFile_Close
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  LogFile_Exit
//
//  Description
//
//      Close the log file and end the session.
//
//  Arguments
//
//      Void
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void LogFile_Exit(void)
{
time_t ltime;

    //
    //  write a time stamp
    //
    time(&ltime);

    fprintf(flogfile, "End time: %s\n\n", ctime(&ltime));

    LogFile_Close();
}
//  function -- LogFile_Exit
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  LogFile_Create
//
//  Description
//
//      Create the log file.
//
//  Arguments
//
//      Void
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void LogFile_Create(void)
{
time_t ltime;


    flogfile = fopen("logfile.txt", "w");
    if (flogfile == NULL) {
        fprintf(stderr,"ERROR: LogFile_Create: Failure\n");
        exit(1);
	}

    //
    //  write a time stamp
    //
    time(&ltime);

    fprintf(flogfile, "Log File: attrk\n");
    fprintf(flogfile, "%s\n\n", ctime(&ltime));

}
//  function -- LogFile_Create
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  LogFile_Printf
//
//  Description
//
//      Printf an "fprintf-formated" string to the log file.
//
//  Arguments
//
//      fmt
//
//          A C "printf-formated" string followed by a variable argument
//          list.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void LogFile_Printf(char *fmt, ...)
{
va_list vl;
int tabs;

    //
    //  vl is a pointer to arguments on the function stack
    //  va_start points "vl" to the first argument that comes after "fmt"
    //
    va_start(vl, fmt);

    //
    //	indent according to "numtabs"
    //
    tabs = numlogtabs;
    while (tabs > 0) {
        fprintf(flogfile, "\t");
        tabs--;
	}

    //
    //	now print the formatted data
    //
    vfprintf(flogfile, fmt, vl);

    va_end(vl);

    fflush(flogfile);
}
//  function -- LogFile_Printf
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  v_LogFile_Printf
//
//  Description
//
//      Special version of LogFile_Printf called from Error_Printf.
//
//  Arguments
//
//      fmt
//
//          A C "printf-formated" string followed by a variable argument
//          list.
//
//      va_list
//
//          Variable argument list (see man page for stdarg)
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void v_LogFile_Printf(char *fmt, va_list vl)
/*
	Special version of LogFile_Printf called from Error_Printf.
*/
{
    if (flogfile)
    {
        vfprintf(flogfile, fmt, vl);
        fflush(flogfile);
    }
}
//  function -- v_LogFile_Printf
//  -------------------------------------------------------------------------


