/**
 * C code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MIDICONTROL_H_INCLUDED
#define MIDICONTROL_H_INCLUDED


struct BankSel {
    int number;
    int value;
    struct BankSel *next;
};
struct BankSel *Create_BankSel(int number, int value);
void Delete_BankSel(struct BankSel *bs);


struct Bank {
    struct BankSel *msg;
    struct Bank *next;
};
struct Bank *Create_Bank(struct BankSel *bs);
void Delete_Bank(struct Bank *bk);



struct MidiControl
{
	struct RawMidi *mididev;
	int useintclock;
	int isrt;
	int bp;
	int bcount;
	int rtcfreq;

	// Last sec/usec we sent a sync signal.
	int last_sec;
	int last_usec;
	int last_diff;

	// RTC device.
	struct RealTimeClock *rtc;

	// Our cheezy MIDI tick counter (24ppq for now).
	int midiclockcounter;

	// Ugly hacked state information.
	int wasplaying;

    int alsa;

    struct MidiControl *next;
};

struct MidiControl *Create_MidiControl(int useintclock, int isrt, int bp,
                                       int bcount, int rtcfreq,
                                       int midiclockcounter, int wasplaying);
void Delete_MidiControl(struct MidiControl *mctrl);
int mctrl_openMidiDevice(struct MidiControl *mctrl,  const char *filename);
void mctrl_doNote(struct MidiControl *mctrl,  struct SongChannel *chan,
                         struct SongPattern *pat, unsigned int beat);
void mctrl_doProgramChange(struct MidiControl *mctrl, struct SongChannel *chan);
void mctrl_doVolumeChange(struct MidiControl *mctrl, struct SongChannel *chan);
void mctrl_shutUp(struct MidiControl *mctrl);
void mctrl_rtcTimeCheck(struct MidiControl *mctrl);
void mctrl_midiTimeCheck(struct MidiControl *mctrl);
int mctrl_isMidiDeviceOpen(struct MidiControl *mctrl, int port);
int mctrl_isRealtime(struct MidiControl *mctrl);
void mctrl_useInternalClock(struct MidiControl *mctrl);
void mctrl_useExternalClock(struct MidiControl *mctrl);
int mctrl_usingInternalClock(struct MidiControl *mctrl);
int mctrl_beatPulse(struct MidiControl *mctrl);
void mctrl_setRTCFrequency(struct MidiControl *mctrl, int newfreq);
int thread_main(struct MidiControl *mctrl);
void mctrl_doCurrentNotes(struct MidiControl *mctrl);
void mctrl_doCurrentMessages(struct MidiControl *mctrl);
void mctrl_doControlChange(struct MidiControl *mctrl, int ichan,
                           int ctrlnumber, int value);
void mctrl_doControlSetupChange(struct MidiControl *mctrl,
                                struct SongChannel *chan);
void mctrl_doSetupMessages(struct MidiControl *mctrl, struct SongChannel *schan);

#endif // MIDICONTROL_H_INCLUDED
