/**
 * C code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RAWMIDI_H_INCLUDED
#define RAWMIDI_H_INCLUDED

#define MIDI_BUFFSIZE       1024
#define MIDI_SAFEPOS(p)     ((p)%MIDI_BUFFSIZE)

struct RawMidi {

#ifdef ALSA_DRIVER
    snd_rawmidi_t *ahandle_o;
    snd_rawmidi_t *ahandle_i;
#endif

	int midi_fd;
	int buffpos;
	unsigned char buff[ MIDI_BUFFSIZE ];
    int port;
    int alsa;
    struct RawMidi *next;

};

int mdev_isOpen(struct RawMidi *rmid);
struct RawMidi *Create_RawMidi(const char *mididev, int mode, int alsa);
void Delete_RawMidi(struct RawMidi *rmid);
struct RawMidi *Find_Last_RawMidi(struct RawMidi *mididev_head);
struct RawMidi *Find_Nth_RawMidi(struct RawMidi *mididev_head, int number);
void mdev_noteOn(struct RawMidi *mididev,  unsigned int chan,
                 unsigned int port, unsigned int note, unsigned int vel);
void mdev_noteOff(struct RawMidi *mididev,  unsigned int chan,
                  unsigned int port, unsigned int note, unsigned int vel);
void mdev_polyphonicAftertouch(struct RawMidi *mididev, unsigned int chan,
                               unsigned int port, unsigned int note,
                               unsigned int amt);
void mdev_programChange(struct RawMidi *mididev, unsigned int chan,
                        unsigned int port, unsigned int prog);
void mdev_controlChange(struct RawMidi *mididev, unsigned int chan,
                        unsigned int port, unsigned int num, unsigned int val);
void mdev_channelAftertouch(struct RawMidi *mididev, unsigned int chan,
                            unsigned int port, unsigned int amt);
void mdev_pitchWheel(struct RawMidi *mididev, unsigned int chan,
                     unsigned int port, unsigned int amt);
void mdev_allSoundOff(struct RawMidi *mididev, int chan, int port);
void mdev_resetAllControllers(struct RawMidi *mididev, int chan, int port);
void mdev_setLocal(struct RawMidi *mididev, int chan, int port, int on);
void mdev_allNotesOff(struct RawMidi *mididev, int chan, int port);
void mdev_allNotesOff_all(struct RawMidi *mididev);
void mdev_syncStart(struct RawMidi *mididev);
void mdev_syncStop(struct RawMidi *mididev);
void mdev_syncContinue(struct RawMidi *mididev);
void mdev_syncTick(struct RawMidi *mididev);
int mdev_readMessage(struct RawMidi *mididev, int chan);
void mdev_flush(struct RawMidi *mididev);
void mdev_shutUp(struct RawMidi *mididev);

#endif // RAWMIDI_H_INCLUDED
