/**
 * Copyright (C) 2009 Alfred Steffens Jr.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define REVENT_MAX_IPARMS           5
#define REVENT_MAX_NAME_LENGTH      64

//
//  Event Types
//
#define REVENT_NONE                 0
#define REVENT_QUIT                 1
#define REVENT_CHMUTE1              2
#define REVENT_CHMUTE2              3
#define REVENT_CHMUTE3              4
#define REVENT_CHMUTE4              5
#define REVENT_CHMUTE5              6
#define REVENT_CHMUTE6              7
#define REVENT_CHMUTE7              8
#define REVENT_CHMUTE8              9
#define REVENT_SONG_START           10
#define REVENT_SONG_RWSTART         11
#define REVENT_SONG_STOP            12
#define REVENT_SONG_REWIND          13
#define REVENT_KEY_DOWN             14
#define REVENT_KEY_UP               15
#define REVENT_KEY_LEFT             16
#define REVENT_KEY_RIGHT            17
#define REVENT_SHIFT_KEY_DOWN       18
#define REVENT_SHIFT_KEY_UP         19
#define REVENT_SHIFT_KEY_LEFT       20
#define REVENT_SHIFT_KEY_RIGHT      21
#define REVENT_MIDICHANNEL_UP       22
#define REVENT_MIDICHANNEL_DOWN     23
#define REVENT_VOLUME_UP            24
#define REVENT_VOLUME_DOWN          25
#define REVENT_VELOCITY_UP          26
#define REVENT_VELOCITY_DOWN        27
#define REVENT_PATTERN_UP           28
#define REVENT_PATTERN_DOWN         29
#define REVENT_OCTAVE_UP            30
#define REVENT_OCTAVE_DOWN          31
#define REVENT_SEMITONE_UP          32
#define REVENT_SEMITONE_DOWN        33
#define REVENT_SONGPOSITION_UP      34
#define REVENT_SONGPOSITION_DOWN    35
#define REVENT_TOGGLE_SLIDE         36
#define REVENT_NOTE_EMPTY           37
#define REVENT_NOTE_C__LO           38
#define REVENT_NOTE_CS_LO           39
#define REVENT_NOTE_D__LO           40
#define REVENT_NOTE_DS_LO           41
#define REVENT_NOTE_E__LO           42
#define REVENT_NOTE_F__LO           43
#define REVENT_NOTE_FS_LO           44
#define REVENT_NOTE_G__LO           45
#define REVENT_NOTE_GS_LO           46
#define REVENT_NOTE_A__LO           47
#define REVENT_NOTE_AS_LO           48
#define REVENT_NOTE_B__LO           49
#define REVENT_NOTE_C__HI           50
#define REVENT_NOTE_CS_HI           51
#define REVENT_NOTE_D__HI           52
#define REVENT_NOTE_DS_HI           53
#define REVENT_NOTE_E__HI           54
#define REVENT_NOTE_F__HI           55
#define REVENT_NOTE_FS_HI           56
#define REVENT_NOTE_G__HI           57
#define REVENT_NOTE_GS_HI           58
#define REVENT_NOTE_A__HI           59
#define REVENT_NOTE_AS_HI           60
#define REVENT_NOTE_B__HI           61
#define REVENT_NOTE_C__HX           62
#define REVENT_NOTE_CS_HX           63
#define REVENT_NOTE_D__HX           64
#define REVENT_NOTE_DS_HX           65
#define REVENT_NOTE_E__HX           66
#define REVENT_TEMPO                67
#define REVENT_LOOP_ON              68
#define REVENT_LOOP_POSITION        69
#define REVENT_LOOP_DESTINATION     70
#define REVENT_START_POSITION       71
#define REVENT_SAVE_SONG            72
#define REVENT_LOAD_SONG            73
#define REVENT_TOGGLE_SYNC          74
#define REVENT_SAVE_COPY            75
#define REVENT_PASTE_COPY           76
#define REVENT_CHANNEL_NAME         77
#define REVENT_PATTERN_NAME         78
#define REVENT_NOTE_OFF             79
#define REVENT_TOGGLE_ENDNOTE       80
#define REVENT_MIDI_PORT            81
#define REVENT_MIDI_PATCH           82
#define REVENT_SHOW_INFO            83
#define REVENT_SWITCH_MODE          84
#define REVENT_EXT_LOOP_POS         85
#define REVENT_EXT_LOOP_DEST        86
#define REVENT_EXT_MIDI_FLUSH       87
#define REVENT_EXT_MIDI_CONTROL     88
#define REVENT_EXT_VELOCITY_RESET   89
#define REVENT_EXT_VELOCITY_CHANGE  90
#define REVENT_EXT_PATTERN_SET      91
#define REVENT_MAX_RV_NAMES         REVENT_EXT_PATTERN_SET



struct RecordEvent {
    int type;
    int has_iParams;
    int iParams[REVENT_MAX_IPARMS];
    unsigned long index;        // count from 0
    struct RecordEvent *next;
};

struct RecordEvent *Create_RecordEvent(unsigned long index, int type);
void Delete_RecordEvent(struct RecordEvent *revent);
struct RecordEvent *Revent_Array_From_Chain(struct RecordEvent *revent_head);
struct RecordEvent *Attach_Revent(struct RecordEvent *curr_re,
                                  unsigned long beatcount, int type,
                                  int *iParams);
struct RecordEvent *Read_Events_File(char *filename);
int Write_Events_File(char *filename, struct RecordEvent *revent);
int Write_Combined_Events_File(char *filename, struct RecordEvent *revent1,
                               struct RecordEvent *revent2);

