/**
 * C-code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SCREEN_H_INCLUDED
#define SCREEN_H_INCLUDED

struct ModeState {
	int cursorrow;
	int scrolltop;
};


#define HEADERSIZE           6
#define BOTTOMSIZE           9

#define DEFAULT_COLOR                   0
#define CURSOR_COLOR                    1
#define HILIGHT_COLOR                   2
#define HILIGHTCURSOR_COLOR             3
#define MUTEDCHANNEL_COLOR              4
#define REALTIMELIGHT_COLOR             5
#define PLAYPOSITION_COLOR              6
#define LOOPHILIGHT_COLOR               7
#define LOOPINACTIVE_COLOR              8

#define SL_KEY_F10              0x1000
#define SL_KEY_F11              0x1001
#define SL_KEY_F12              0x1002
#define SL_KEY_SHIFTLEFT        0x1003
#define SL_KEY_SHIFTRIGHT       0x1004
#define SL_KEY_SHIFTUP          0x1005
#define SL_KEY_SHIFTDOWN        0x1006

struct Kmarker {
    int row;
    int col;
};

struct Screen {
    void **bars;
	struct PatBar **patbars;
	struct SongBar **songbars;
    struct Song *curr_song;
	int pos;
	int octave;
	int patmode;
	int curcol;
    int scrsize;
    int mark_chan1, mark_chan2;
    int mark_row1, mark_row2;
    int *copybuffer;
    int copyrows, copycols;
	struct ModeState curstate;
	struct ModeState patstate;
	struct ModeState songstate;
	struct SongPattern *copyPattern;
    struct Kmarker kmark[26];
};

int Init_Screen(struct Screen *screen, struct Song *song);
struct Screen *Create_Screen(struct Song *song);
void Delete_Screen(struct Screen *screen);
void screen_switchMode(struct Screen *screen);
void screen_refresh(struct Screen *screen);
int screen_pollScreen(struct Screen *screen);
int screen_pollScreenRec(struct Screen *screen);
void screen_printHeader(struct Screen *screen);
void screen_printSidebar(struct Screen *screen);
void screen_printHelp(struct Screen *screen);
void screen_showStatusMessage(struct Screen *screen, char *newmsg);
void screen_moveCursorPosition(struct Screen *screen, int newrow, int newcol);
void screen_setPosition(struct Screen *screen,  int newpos);
void screen_inputTempo(struct Screen *screen);
void screen_inputTempoRec(struct Screen *screen);
void screen_inputTempoRE(struct Screen *screen);
void screen_saveSong(struct Screen *screen);
void screen_saveSongAscii(struct Screen *screen);
void screen_saveSongAscoreWrite(struct Screen *screen);
void screen_loadSong(struct Screen *screen);
void screen_loadSongAscii(struct Screen *screen);
void screen_enterPatName(struct Screen *screen);
void screen_enterChanName(struct Screen *screen);
void screen_toggleSync(struct Screen *screen);
void screen_saveCopy(struct Screen *screen);
void screen_pasteCopy(struct Screen *screen);
void screen_setNote(struct Screen *screen, int newnote);
void screen_toggleEndNote(struct Screen *screen);
void screen_editMidiPort(struct Screen *screen);
void screen_editMidiPortRec(struct Screen *screen);
void screen_editMidiPortRE(struct Screen *screen);
void screen_editMidiPatch(struct Screen *screen);
void screen_editMidiPatchRec(struct Screen *screen);
void screen_editMidiPatchRE(struct Screen *screen);
void screen_showMIDIinfo(struct Screen *screen);
int screen_pollRecord(struct Screen *screen);
void SLang_message(char *msg);
void screen_loadSession(struct Screen *screen);
void screen_setLoopPositionRE(struct Screen *screen);
void screen_setLoopDestinationRE(struct Screen *screen);
void screen_exitWaitmsg(struct Screen *screen, int count);
#
void screen_extCommandRec(struct Screen *screen);
void screen_extCmdMidiFlushRec(struct Screen *screen);
void screen_extCmdMidiFlushRE(struct Screen *screen);
void screen_extCmdMidiControlRec(struct Screen *screen, int n1, int n2);
void screen_extCmdMidiControlRE(struct Screen *screen);
void screen_extCmdVelocityRsRec(struct Screen *screen);
void screen_extCmdVelocityRec(struct Screen *screen, int velocity);
void screen_extCmdVelocityRsRE(struct Screen *screen);
void screen_extCmdVelocityRE(struct Screen *screen);
void screen_extCmdPatternSetRec(struct Screen *screen, int ipat);
void screen_extCmdPatternSetRE(struct Screen *screen);
void screen_saveSongAsciiWrite(struct Screen *screen);
void screen_extCmdWriteRec(struct Screen *screen);
void screen_extCmdWriteARec(struct Screen *screen);
void screen_extCmdEdMarkMRec(struct Screen *screen);
void screen_extCmdEdMarkNRec(struct Screen *screen);
void screen_extCmdEdMarkClearRec(struct Screen *screen);
void screen_extCmdEdCopyRec(struct Screen *screen);
void screen_extCmdEdPasteRec(struct Screen *screen);
void screen_extCmdEdCutRec(struct Screen *screen);
void screen_extCmdEdDelRec(struct Screen *screen);
void screen_extCmdEdYankRec(struct Screen *screen);
void screen_extCmdLoadSession(struct Screen *screen, const char *fname);
void screen_extCmdTogglePlay(struct Screen *screen);
void screen_extCmdPlayOnOff(struct Screen *screen, const char *arg);

#endif // SCREEN_H_INCLUDED
