/**
 * C-code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdarg.h>
#include <zlib.h>
#include <defs.h>
#include <logprintf.h>
#include <errorprintf.h>
#include <xmltools.h>
#include <song.h>
#include <midicontrol.h>

int PAT_BLANK  = 0;
int PAT_FULL   = 1;
int INDEX_DONE = -1;
char blank_name[] = "";
extern int numlogtabs;
extern struct MidiControl *p_MidiControl;
extern struct Bank *p_Banks;

short pchnote[14][12] = {
    {0,0,0,0,0,0,0,0,0,0,0,0},      // 0
    {0,0,0,0,0,0,0,0,0,0,0,0},      // 1
    {0,0,0,0,0,0,0,0,0,0,0,0},      // 2
    {   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11},      // 3
    {  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23},      // 4
    {  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35},      // 5
    {  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47},      // 6
    {  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59},      // 7
    {  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71},      // 8
    {  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83},      // 9
    {  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95},      // 10
    {  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107},      // 11
    { 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119},      // 12
    { 120, 121, 122, 123, 124, 125, 126, 127, 127, 127, 127, 127}       // 13
};


//  -------------------------------------------------------------------------
//  Function    --  Init_SongPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int Init_SongPattern(struct SongPattern *spat, int id)
{
    ERROR_NULL(spat,-1);

    spat->note = NULL;
    spat->slide = NULL;
    spat->vol = NULL;

	spat->note = (int *)malloc( sizeof(int) * SONGPATLEN );
    if (spat->note == NULL)
    {
        Error_Printf("E250: spat_Init_SongPattern: mem alloc on"
                     " note array\n");
        goto Error;
    }

	spat->slide = (int *)malloc( sizeof(int) * SONGPATLEN );
    if (spat->slide == NULL)
    {
        Error_Printf("E251: spat_Init_SongPattern: mem alloc on "
                     " slide array\n");
        goto Error;
    }

	spat->vol = (int *)malloc( sizeof(int) * SONGPATLEN );
    if (spat->vol == NULL)
    {
        Error_Printf("E252: spat_Init_SongPattern: mem alloc on "
                     " vol array\n");
        goto Error;
    }

    spat->id = id;  // number 1..N
    spat->name[0] = 0x00;
	spat->endnote = ( (SONGPATLEN-1) < 32 ?  (SONGPATLEN-1) : 32 );
    spat->asc_noteoff = 0;
    spat->next = NULL;

    spat_clear(spat);
    return 0;

Error:
    if (spat->note)
        free(spat->note);
    if (spat->slide)
        free(spat->slide);
    if (spat->vol)
        free(spat->vol);
    return -1;
}
//  function -- Init_SongPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Create_SongPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongPattern *Create_SongPattern(int id)
{
    struct SongPattern *spat;

    spat = (struct SongPattern *)malloc( sizeof(struct SongPattern) );
    if (spat == NULL)
    {
        Error_Printf("E253: Create_SongPattern: mem alloc\n");
        return NULL;
    }

    if (Init_SongPattern(spat, id))
        return NULL;

    return spat;
}
//  function -- Create_SongPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Delete_SongPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void Delete_SongPattern(struct SongPattern *spat)
{
    struct SongPattern *pN, *pS;

    if (spat == NULL)
        return;

    pS = spat;
    while (pS)
    {
        pN = pS;
        pS = pS->next;
        if (pN->note)
            free(pN->note);
        if (pN->slide)
            free(pN->slide);
        if (pN->vol)
            free(pN->vol);
        free(pN);
    }
}
//  function -- Delete_SongPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_clear
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_clear(struct SongPattern *spat)
{
    int i;

    ERROR_NULL_VOID(spat);

    for (i = 0; i < SONGPATLEN; ++i)
    {
        spat->note[i] = SONG_NOTEEMPTY;
        spat->slide[i] = M_FALSE;
        spat->vol[i] = DEFAULT_VELOCITY;
        spat->name[0] = 0x00;
	}
}
//  function -- spat_clear
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_isBlank
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_isBlank(struct SongPattern *spat)
{
    int i;

    if (spat == NULL)
        return M_TRUE;

    for (i = 0; i < SONGPATLEN; ++i)
    {
        if (spat->note[i] != SONG_NOTEEMPTY || spat->slide[i] != M_FALSE
                    || spat->vol[i] != 64 || strlen(spat->name) > 0 )
        {
            return M_FALSE;
        }
    }
    return M_TRUE;
}
//  function -- spat_isBlank
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_isBlankorOff
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_isBlankorOff(struct SongPattern *spat)
{
    int i;

    if (spat == NULL)
        return M_TRUE;

    for (i = 0; i < SONGPATLEN; ++i)
    {
        if (spat->note[i] == SONG_NOTEOFF)
            continue;

        if (spat->note[i] == SONG_NOTEEMPTY)
            continue;

        return M_FALSE;
    }

    return M_TRUE;
}
//  function -- spat_isBlankorOff
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  spat_copyTo
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_copyTo(struct SongPattern *spat,  struct SongPattern *pat)
{
    int i;

    ERROR_NULL_VOID(spat);
    ERROR_NULL_VOID(pat);

    for (i = 0; i < SONGPATLEN; ++i)
    {
        pat->note[i] = spat->note[i];
        pat->slide[i] = spat->slide[i];
        pat->vol[i] = spat->vol[i];
    }
    pat->endnote = spat->endnote;
}
//  function -- spat_copyTo
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_setVolume
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_setVolume(struct SongPattern *spat,  int beat, int value)
{
    ERROR_NULL_VOID(spat);

    if (beat < SONGPATLEN)
    {
        if (value < 0)
            value = 0;
        else if (value > 127)
            value = 127;
        spat->vol[beat] = value;
    }
}
//  function -- spat_setVolume
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_setName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_setName(struct SongPattern *spat, const char *newname)
{
    ERROR_NULL_VOID(spat);
    ERROR_NULL_VOID(newname);

    strncpy(spat->name, newname, NAME_LENGTH);
    spat->name[NAME_LENGTH] = 0x00;
}
//  function -- spat_setName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_transposeNotes
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_transposeNotes(struct SongPattern *spat, int trans)
{
    int i;

    ERROR_NULL_VOID(spat);

    for (i = 0; i < SONGPATLEN; ++i)
    {
        if (spat->note[i] != SONG_NOTEEMPTY
                          && spat->note[i] != SONG_NOTEOFF)
        {
            spat->note[i] += trans;
            if (spat->note[i] > 128 )
            {
                spat->note[i] = 128;
            }
            else if (spat->note[i] < -1)
            {
                spat->note[i] = -1;
            }
        }
	}
}
//  function -- spat_transposeNotes
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  spat_getIndex
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_getIndex(struct SongPattern *spat)
{
    ERROR_NULL(spat,0);
    return spat->id;
}
//  function -- spat_getIndex
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_getNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_getNote(struct SongPattern *spat, int beat)
{
    ERROR_NULL(spat,SONG_NOTEEMPTY);

    if (beat < SONGPATLEN)
         return spat->note[beat];
    else
        return SONG_NOTEEMPTY;
}
//  function -- spat_getNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_setNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_setNote(struct SongPattern *spat, int beat, int value)
{
    ERROR_NULL_VOID(spat);

    if (beat < SONGPATLEN)
    {
        if (beat <= spat->endnote)
            spat->note[beat] = value;
    }
}
//  function -- spat_setNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_getSlide
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_getSlide(struct SongPattern *spat, int beat)
{
    ERROR_NULL(spat,0);

    if (beat < SONGPATLEN)
        return spat->slide[beat];
    else
        return 0;
}
//  function -- spat_getSlide
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_toggleSlide
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_toggleSlide(struct SongPattern *spat, int beat)
{
    ERROR_NULL_VOID(spat);

    if (beat < SONGPATLEN)
        spat->slide[beat] = !spat->slide[beat];
}
//  function -- spat_toggleSlide
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_getVolume
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_getVolume(struct SongPattern *spat, int beat)
{
    ERROR_NULL(spat,0);

    if (beat < SONGPATLEN)
        return spat->vol[beat];
    else
        return 0;
}
//  function -- spat_getVolume
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_getName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
const char *spat_getName(struct SongPattern *spat)
{
    const char *pch;

    ERROR_NULL(spat,NULL);

    pch = (const char *)spat->name;
    return pch;
}
//  function -- spat_getName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_getEndNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int spat_getEndNote(struct SongPattern *spat)
{
    ERROR_NULL(spat,0);

    return spat->endnote;
}
//  function -- spat_getEndNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  spat_setEndNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void spat_setEndNote(struct SongPattern *spat, int enote)
{
    ERROR_NULL_VOID(spat);

    if (enote == spat->endnote)
    {
        spat->endnote = SONGPATLEN-1;
    }
    else
    {
        spat->endnote = enote;
    }
}
//  function -- spat_setEndNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Create_SChanMidiCtrl
//
//  Description
//
//      Constructor for the SChanMidiCtrl struct instance
//
//  Arguments
//
//      Void
//
//  Return
//
//      Allocated, initialized pointer to a new SChanMidiCtrl struct instance.
//
//  -------------------------------------------------------------------------
struct SChanMidiCtrl *Create_SChanMidiCtrl(void)
{
    struct SChanMidiCtrl *sctrl;

    sctrl = (struct SChanMidiCtrl *)malloc( sizeof(struct SChanMidiCtrl) );
    if (sctrl == NULL)
    {
        Error_Printf("E253: Create_SChanMidiCtrl: mem alloc\n");
        return NULL;
    }
    sctrl->chg_ready = 0;
    sctrl->control_number = 0;
    sctrl->control_value = 0;
    sctrl->next = NULL;

    return sctrl;
}
//  function -- Create_SChanMidiCtrl
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Delete_SChanMidiCtrl
//
//  Description
//
//      Deletes (frees) the chain of SChanMidiCtrl struct instances with
//      head *sctrl.
//
//  Arguments
//
//      sctrl
//
//          Pointer to struct instance of SChanMidiCtrl and is allowed
//          to be the head of a linked list.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void Delete_SChanMidiCtrl(struct SChanMidiCtrl *sctrl)
{
    struct SChanMidiCtrl *pN, *pS;

    if (sctrl == NULL)
        return;

    pS = sctrl;
    while (pS)
    {
        pN = pS;
        pS = pS->next;
        free(pN);
    }
}
//  function -- Delete_SChanMidiCtrl
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function for SongChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Init_SongChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int Init_SongChannel(struct SongChannel *schan)
{
    int i;

    ERROR_NULL(schan,-1);

    schan->patterns = (struct SongPattern **)malloc(
                       sizeof(struct SongPattern *) * SONGMAXPAT );
    if (schan->patterns == NULL)
    {
        Error_Printf("E254: Init_SongChannel: mem alloc on"
                     " patterns\n");
        return -1;
    }

    for (i = 0; i < SONGMAXPAT; ++i)
    {
        schan->patterns[i] = NULL;
    }

    for (i = 0; i < SONGMAXPAT; ++i)
    {
        schan->patterns[i] = Create_SongPattern(i+1);
        if (schan->patterns[i] == NULL)
        {
            goto Error;
        }
    }

	schan->songdata = (int *)malloc( SONGLEN * sizeof(int) );
    if (schan->songdata == NULL)
    {
        Error_Printf("E255: Init_SongChannel: mem alloc on"
                     " songdata\n");
        goto Error;
    }

    schan->name[0] = 0x00;
    schan->ctrl_msg = NULL;
    schan->next = NULL;
    return 0;

Error:

    for (i = 0; i < SONGMAXPAT; ++i)
    {
        Delete_SongPattern(schan->patterns[i]);
    }
    free(schan->patterns);
    return -1;
}
//  function -- Init_SongChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Create_SongChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongChannel *Create_SongChannel(void)
{
    struct SongChannel *schan;

    schan = (struct SongChannel *)malloc( sizeof(struct SongChannel) );
    if (schan == NULL)
    {
        Error_Printf("E256: Create_SongChannel: mem alloc\n");
        return NULL;
    }

    if (Init_SongChannel(schan))
        return NULL;

    return schan;
}
//  function -- Create_SongChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Delete_SongChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void Delete_SongChannel(struct SongChannel *schan)
{
    struct SongChannel *pN, *pS;

    if (schan == NULL)
        return;

    pS = schan;
    while (pS)
    {
        int i;

        pN = pS;
        pS = pS->next;
        if (pN->patterns)
        {
            for (i = 0; i < SONGMAXPAT; ++i)
            {
                Delete_SongPattern(pN->patterns[i]);
            }
            free(pN->patterns);
        }
        if (pN->songdata)
            free(pN->songdata);
        Delete_SChanMidiCtrl(pN->ctrl_msg);
        free(pN);
    }
}
//  function -- Delete_SongChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_SetSongPointer
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_SetSongPointer(struct SongChannel *schan, void *song)
{
    int i;

    ERROR_NULL_VOID(schan);
    ERROR_NULL_VOID(song);

    schan->curr_song = song;

    for (i = 0; i < SONGMAXPAT; ++i)
    {
        struct SongPattern *spat;

        spat = schan->patterns[i];
        if (spat)
            spat->curr_song = song;
    }
}
//  function -- schan_SetSongPointer
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_clear
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_clear(struct SongChannel *schan)
{
    int i;

    ERROR_NULL_VOID(schan);

    schan->midichannel = 0;
    schan->midiport = 0;
    schan->volume = 75;
    schan->devnum = 1;
    schan->lastnote = 0;
    schan->mute = M_FALSE;
    schan->name[0] = 0x00;
    schan->prognum = 0;
    schan->banknum = 0;
    schan->prchg_ready = 0;
    schan->bnchg_ready = 0;
    schan->vlchg_ready = 0;
    schan->midisetup_ready = 0;

    for (i = 0; i < SONGLEN; ++i)
        schan->songdata[i] = 0;
    for (i = 0; i < SONGMAXPAT; ++i)
        spat_clear(schan->patterns[i]);
}
//  function -- schan_clear
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setVolume
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setVolume(struct SongChannel *schan, int newvol)
{
    ERROR_NULL_VOID(schan);

    if (newvol < 1)
        schan->volume = 0;
    else if (newvol > 128 )
        schan->volume = 127;
    else
        schan->volume = newvol-1;

    schan->vlchg_ready = 1;
}
//  function -- schan_setVolume
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setMidiChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setMidiChannel(struct SongChannel *schan, int newchan)
{
    ERROR_NULL_VOID(schan);

    if (newchan < 0)
        schan->midichannel = 0;
    else if (newchan > 15)
        schan->midichannel = 15;
    else
        schan->midichannel = newchan;
}
//  function -- schan_setMidiChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setMidiPort
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setMidiPort(struct SongChannel *schan, int newport)
{
    ERROR_NULL_VOID(schan);

    schan->midiport = newport-1;
    if (schan->midiport < 0)
        schan->midiport = 0;
}
//  function -- schan_setMidiPort
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setSongPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setSongPattern(struct SongChannel *schan, int index, int patid)
{
    ERROR_NULL_VOID(schan);

    //
    //  patid is stored as a number 1..N
    //
    if (patid > SONGMAXPAT)
        patid = SONGMAXPAT;

    //
    //  Allow the patid to be zero (no pattern)
    //
    if (patid < 0)
        patid = 0;

    schan->songdata[index] = patid;
}
//  function -- schan_setSongPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setName(struct SongChannel *schan, const char *newname)
{
    ERROR_NULL_VOID(schan);

    if (newname && strlen(newname))
    {
        strncpy(schan->name, newname, NAME_LENGTH);
        schan->name[NAME_LENGTH] = 0x00;
    }
}
//  function -- schan_setName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getProgramNumber
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getProgramNumber(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->prognum+1;
}
//  function -- schan_getProgramNumber
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setProgramNumber
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setProgramNumber(struct SongChannel *schan, int newprg)
{
    ERROR_NULL_VOID(schan);

    schan->prognum = newprg-1;
    if (schan->prognum < 0)
        schan->prognum = 0;
    
    schan->prchg_ready=1;
}
//  function -- schan_setProgramNumber
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getBankNumber
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getBankNumber(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    //
    //  In the SongChannel struct, bank is numbered 0....N
    //
    return schan->banknum+1;
}
//  function -- schan_getBankNumber
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setBankNumber
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setBankNumber(struct SongChannel *schan, int newbnk)
{
    ERROR_NULL_VOID(schan);

    //
    //  In the SongChannel struct, bank is numbered 0....N
    //
    schan->banknum = newbnk-1;

    if (schan->banknum >= 0)
        schan->bnchg_ready=1;
}
//  function -- schan_setBankNumber
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getMidiChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getMidiChannel(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->midichannel;
}
//  function -- schan_getMidiChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getMidiPort
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getMidiPort(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->midiport;
}
//  function -- schan_getMidiPort
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getDevice
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getDevice(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->devnum;
}
//  function -- schan_getDevice
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getLastNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getLastNote(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->lastnote;
}
//  function -- schan_getLastNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_setLastNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_setLastNote(struct SongChannel *schan, int last)
{
    ERROR_NULL_VOID(schan);
    schan->lastnote = last;
}
//  function -- schan_setLastNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
const char *schan_getName(struct SongChannel *schan)
{
    ERROR_NULL(schan,NULL);
    return (const char *)schan->name;
}
//  function -- schan_getName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_isMuted
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_isMuted(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->mute;
}
//  function -- schan_isMuted
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_toggleMute
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void schan_toggleMute(struct SongChannel *schan)
{
    ERROR_NULL_VOID(schan);
    schan->mute = !schan->mute;
}
//  function -- schan_toggleMute
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getSongPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongPattern *schan_getSongPattern(struct SongChannel *schan, int index)
{
    int patid;

    ERROR_NULL(schan,NULL);

    patid = schan->songdata[index];
    if (patid > 0)
        return schan->patterns[patid-1];

    return NULL;
}
//  function -- schan_getSongPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getChannelPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongPattern *schan_getChannelPattern(struct SongChannel *schan, int index)
{
    ERROR_NULL(schan,NULL);

    assert(index >= 0);
    assert(index < SONGMAXPAT);
    return schan->patterns[index];
}
//  function -- schan_getChannelPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  schan_getVolume
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int schan_getVolume(struct SongChannel *schan)
{
    ERROR_NULL(schan,0);
    return schan->volume+1;
}
//  function -- schan_getVolume
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  Init_Song
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int Init_Song(struct Song *song)
{
    int i;

    ERROR_NULL(song,-1);

    song->playing = 0;
    song->curpat = 0;
    song->curbeat = 0;
    song->beatcount = 0;
    song->startpat = 0;
    song->loopactive = 0;
    song->tempo = 0;
    song->usecpb = 0;
    song->looppos = 0;
    song->loopdest = 0;
    song->re_record = 0;
    song->re_play = 0;
    song->filename[0] = 0x00;
    song->channels = NULL;
    song->next = NULL;


    //
    //  allocate memory for the array of SongChannel structs
    //
	song->channels = (struct SongChannel **)malloc(
                      sizeof(struct SongChannel *) * SONGMAXCHAN
                      );
    if (song->channels == NULL)
    {
        Error_Printf("E257: Create_Song: mem alloc on channels\n");
        goto Error;
    }

    //
    //  Create each of the SongChannel structs
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        song->channels[i] = NULL;
    }
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        song->channels[i] = Create_SongChannel();
        if (song->channels[i] == NULL)
            goto Error;
    }


    song_SetSongPointer(song);
    song_clear(song);

    return 0;

Error:
    if (song->channels)
    {
        for (i = 0; i < SONGMAXCHAN; ++i)
        {
            Delete_SongChannel(song->channels[i]);
        }
    }
    free(song);
    return -1;
}
//  function -- Init_Song
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Create_Song
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct Song *Create_Song(void)
{
    struct Song *song;

    song = (struct Song *)malloc( sizeof(struct Song) );
    if (song == NULL)
    {
        Error_Printf("E258: Create_Song: mem alloc\n");
        return NULL;
    }

    if (Init_Song(song))
        return NULL;

    return song;
}
//  function -- Create_Song
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Delete_Song
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void Delete_Song(struct Song *song)
{
    int i;
    struct Song *pN, *pS;

    if (song == NULL)
        return;

    pS = song;
    while (pS)
    {
        pN = pS;
        pS = pS->next;
        if (pN->channels)
        {
            for (i = 0; i < SONGMAXCHAN; ++i)
            {
                Delete_SongChannel(pN->channels[i]);
            }
            free(pN->channels);
        }
        free(pN);
    }
}
//  function -- Delete_Song
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  song_SetSongPointer
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_SetSongPointer(struct Song *song)
{
    int i;

    ERROR_NULL_VOID(song);

    //
    //  Loop through all the channels and set the song pointer
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        schan_SetSongPointer(song->channels[i], (void *)song);
    }
}
//  function -- song_SetSongPointer
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_clear
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_clear(struct Song *song)
{
    int i;

    ERROR_NULL_VOID(song);

    for (i = 0; i < SONGMAXCHAN; ++i)
        schan_clear(song->channels[i]);
    song->startpat = 0;
    song->loopactive = M_TRUE;
    song->looppos = 1;
    song->loopdest = 0;
    song_setTempo(song, DEFAULT_TEMPO);
}
//  function -- song_clear
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_setTempo
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_setTempo(struct Song *song, int newtempo)
{
    ERROR_NULL_VOID(song);

	song->tempo = newtempo;
	song->usecpb = 60 * 1000 * 1000 / newtempo / 8; // 32nd notes, divisor is 8.
}
//  function -- song_setTempo
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getEndNote
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getEndNote(struct Song *song)
{
    int i;
    int min_endnote;

    ERROR_NULL(song,0);

    min_endnote = SONGPATLEN-1;

    for (i = 0; i < SONGMAXCHAN; i++)
    {
        struct SongChannel *curchan;
        struct SongPattern *pat;

        curchan = song_getChannel(song, i);
        if (curchan == NULL)
        {
            Error_Printf("E259: song_getEndNote: NULL channel %d\n", i);
        }

        pat = schan_getSongPattern(curchan, song->curpat);
        if (pat)
        {
            int endnote;
            endnote = spat_getEndNote(pat);
            if (endnote < min_endnote)
                min_endnote = endnote;
        }

    } // for -- channels

    return min_endnote;
}
//  function -- song_getEndNote
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_incrBeat
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_incrBeat(struct Song *song)
{
    ERROR_NULL_VOID(song);

	++song->curbeat;
	++song->beatcount;

    if (song->curbeat >= song_getEndNote(song))
    {
        song->curbeat = 0;
        ++song->curpat;

        if (song->curpat == song->looppos && song->loopactive)
            song->curpat = song->loopdest;
    }
}
//  function -- song_incrBeat
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_saveSong
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_saveSong(struct Song *song, const char *filename)
{
	int version = 0;
	int i, j, k, len;
    int maxlen, maxchan;
    gzFile fd;

    ERROR_NULL(song,M_FALSE);

    fd = gzopen( filename, "w" );
    if (fd == 0)
        return M_FALSE;

    // Write version.
    gzwrite(fd, &version, sizeof(int));

    // Write basic song information.

    // Starting position.
    gzwrite(fd, &song->startpat, sizeof(int));

    // Current tempo.
    gzwrite(fd, &song->tempo, sizeof(int));

    // Is looping active.
    gzwrite(fd, &song->loopactive, sizeof(int));

    // Loop position.
    gzwrite(fd, &song->looppos, sizeof(int));

    // Loop destination.
    gzwrite(fd, &song->loopdest, sizeof(int));

	// pattern length
    maxlen = SONGPATLEN;
    gzwrite(fd, &maxlen, sizeof(int));
    // chan length
    maxchan = SONGMAXCHAN;
    gzwrite(fd, &maxchan, sizeof(int));

    // Write each channel.
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        // Write the channel data.
        gzwrite(fd, &(song->channels[i]->midichannel), sizeof(int));
        gzwrite(fd, &(song->channels[i]->volume), sizeof(int));
        gzwrite(fd, &(song->channels[i]->devnum), sizeof(int));
		gzwrite(fd, &(song->channels[i]->mute), sizeof(int));

        // Write the channel name: length, string.
        len = strlen(song->channels[i]->name) + 1;
        gzwrite(fd, &len, sizeof(int));
		gzwrite(fd, song->channels[i]->name, len);

        // Write song pattern indexes.
        for (j = 0; j < SONGLEN; ++j )
        {
            if (song->channels[i]->songdata[j])
            {
                gzwrite(fd, &j, sizeof(int));
                gzwrite(fd, &(song->channels[i]->songdata[j]), sizeof(int));
            }
        }
        gzwrite(fd, &INDEX_DONE, sizeof(int));

        // Now write all the patterns.
        for(j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlank(song->channels[i]->patterns[j]))
            {
                gzwrite(fd, &PAT_BLANK, sizeof(int));
                continue;
            }

            gzwrite( fd, &PAT_FULL, sizeof( int ) );

            // Write the pattern name: length, string.
            len = strlen(song->channels[i]->patterns[j]->name) + 1;
            gzwrite(fd, &len, sizeof(int));
            gzwrite(fd, song->channels[i]->patterns[j]->name, len);

            //	Write the end note (A.S.)
            gzwrite(fd, &song->channels[i]->patterns[j]->endnote, sizeof(int));

            for (k = 0; k < SONGPATLEN; ++k)
            {
                gzwrite(fd, &(song->channels[i]->patterns[j]->note[k]), sizeof(int));
                gzwrite(fd, &(song->channels[i]->patterns[j]->slide[k]), sizeof(int));
                gzwrite(fd, &(song->channels[i]->patterns[j]->vol[k]), sizeof(int));
            }
        }
    }

    // Close the file.
    gzclose(fd);

    // Success!
    return M_TRUE;
}
//  function -- song_saveSong
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  str_TruncRepeats
//
//  Description
//
//      Helper for song_saveSongAscore().  Truncate repeated tokens in
//      certain strings such as duration and velocity.
//
//  Arguments
//
//      str
//
//          Pointer to character string containing the tokens.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void str_TruncRepeats(char *str)
{
    char *pch=NULL, *pch_last=NULL, *pch_first=NULL;
    static char strTmpText[512]; // size=strTmpText[] in saveSongAscore
    int trunc=1;

    if (str == NULL)
        return;

    strcpy(strTmpText, str);
    pch = strtok(strTmpText, " ");
    if (pch)
    {
        pch_first = pch;
        pch_last = pch;
        while (pch)
        {
            //
            //  Any differences show that the string is not uniform
            //
            if (strcmp(pch, pch_last) != 0)
            {
                trunc = 0;
                break;
            }

            pch_last = pch;
            pch = strtok(NULL, " ");
        }
    }

    if (trunc)
    {
        //
        //  Save only the first token
        //
        strcpy(str, " ");
        strcat(str, pch_first);
    }
}
//  function -- str_TruncRepeats
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_saveSongAscore
//
//  Description
//
//      Saves the song in ASCORE format.
//
//  Arguments
//
//      song
//
//          Pointer to the Song struct instance.
//
//      filename
//
//          Pointer to string containing the file name of the new song
//          file.
//
//  Return
//
//      Returns 0 on success, nonzero on error.
//
//  -------------------------------------------------------------------------
int song_saveSongAscore(struct Song *song, const char *filename)
{
	int i, j, k,
        instr_count=0, ascore_channel;
    int tempo;
    int songpos, max_songpos, max_chan;
	FILE *afile;
    static char strLine[MAX_ASCORE_LINE];
    int iLineLength;
    int nCenter;
    struct Bank *curr_bank;

    ERROR_NULL(song,M_FALSE);

    //
    //  Some standard parameters
    //  nCenter is the "center" position in a pattern name
    //
    if (NAME_LENGTH % 2 == 0)
        nCenter = (NAME_LENGTH / 2) - 1;
    else
        nCenter = NAME_LENGTH / 2;

    //
    //  Open (create) the file
    //
    afile = fopen(filename, "w");
    if (afile == NULL)
        return M_FALSE;


    // Write version.
    fprintf(afile, "#\n");
    fprintf(afile, "#    %s\n", filename);
    fprintf(afile, "#    Generated from Attrk ver. %s\n", VERSION);
    fprintf(afile, "#\n");
    fprintf(afile, "#\n");
    fprintf(afile, "BEGIN\n");
    fprintf(afile, "#\n");
    fprintf(afile, "#\n");
    fprintf(afile, "\n");

    //
    //  Bank def statements, if any
    //
    if (p_Banks)
    {
        struct BankSel *curr_bs;

        fprintf(afile, "#\n");
        fprintf(afile, "#   Midi bank switch definitions\n");
        fprintf(afile, "#\n");

        curr_bank = p_Banks;
        while (curr_bank)
        {
            curr_bs = curr_bank->msg;
            if (curr_bs)
            {
                fprintf(afile, "BANKDEF   ");
                while (curr_bs)
                {
                    fprintf(afile, " %d %d", curr_bs->number, curr_bs->value);
                    curr_bs = curr_bs->next;
                }
                fprintf(afile, "\n");
            }
            curr_bank = curr_bank->next;
        }
    }

    tempo = song->tempo;

    //
    //  Write all the pattern definitions first
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        int empty;

        //  only save the channel if non-empty
        empty = M_TRUE;
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlank(song->channels[i]->patterns[j])==M_FALSE)
                empty = M_FALSE;
        }
        if (empty == M_TRUE)
            continue;

        //
        //  Print channel info as a comment
        //
        fprintf(afile, "\n");
        fprintf(afile, "#    ------------------------------------");
        fprintf(afile, "------------------------------------\n");
        fprintf(afile, "#    ------------------------------------");
        fprintf(afile, "------------------------------------\n");
        fprintf(afile, "#\n");
        fprintf(afile, "#\n");
        if (strlen(song->channels[i]->name) > 0)
        {
            fprintf(afile, "#    (from Attrk channel index=%d name=%s)\n", i+1,
                    song->channels[i]->name);
        }
        else
        {
            fprintf(afile, "#    (from Attrk channel index=%d)\n", i+1);
        }
        fprintf(afile, "#\n");
        fprintf(afile, "#\n");

        //
        //  Write the CS2MTX header for this channel
        //  MIDI channel, number counting from 1
        //  Volume, number between 0 and 127
        //  bank, number between 0 and N
        //
        instr_count++;
        ascore_channel = ((song->channels[i]->midichannel % 16)+1) +
                         (song->channels[i]->midiport * 16);
        fprintf(afile, "CS2MTX midi_channel %d instrument %d\n",
                        ascore_channel, instr_count);
        fprintf(afile, "CS2MTX midi_channel %d nshift 0\n",
                        ascore_channel);
        fprintf(afile, "CS2MTX midi_channel %d volume %d\n",
                        ascore_channel, song->channels[i]->volume);
        fprintf(afile, "CS2MTX midi_channel %d bank %d\n",
                        ascore_channel, song->channels[i]->banknum);
        fprintf(afile, "CS2MTX midi_channel %d program %d\n",
                        ascore_channel, song->channels[i]->prognum);
        fprintf(afile, "#\n");
        fprintf(afile, "#\n");
                
        //
        // Now write all the patterns.
        //
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            int have_pattern, endnote;
            static char strTmpText[512];
            static char strInstr[32];
            int idec;
            static char strStart[32];
            float start;
            static char strDur[32];
            float dur;
            static char strKey[32];
            static char key[8];
            static char strVel[32];
            int vel;

            //
            //  Check if there is a valid pattern here
            //
            have_pattern = M_FALSE;
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    
                    if (song->channels[i]->patterns[j]->note[k] != SONG_NOTEOFF)
                        have_pattern = M_TRUE;
                    break;
                }
            }
            if (have_pattern == M_FALSE)
                continue;

            endnote = song->channels[i]->patterns[j]->endnote;

            //
			// Write the pattern name as a comment
            //
            fprintf(afile, "\n");
            fprintf(afile, "#    ------------------------------------");
            fprintf(afile, "------------------------------------\n");
            fprintf(afile, "#\n");
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
            {
			    fprintf(afile, "#    %s    (pattern %d, chan %d)\n",
                    song->channels[i]->patterns[j]->name, j+1, i+1);
            }
            else
            {
			    fprintf(afile, "#    (pattern %d, chan %d)\n", j+1, i+1);
            }
            fprintf(afile, "#\n");

            //
            //  Write the instrument number string
            //
            idec = 0;
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strInstr, "ins_%s",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strInstr, "ins_%s_%d",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strInstr, "ins_%d_%d", i+1, j+1);
            }
            sprintf(strLine, "set %s   ", strInstr);
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (k == endnote)
                    break;

                if (song->channels[i]->patterns[j]->note[k] == SONG_NOTEOFF)
                    continue;

                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    sprintf(strLine, "%s %d.%d", strLine, instr_count,
                            (idec % 10)+1);
                    idec++;
                }
			}
            fprintf(afile, "%s\n", strLine);

            //
            //  Write the start time string
            //  (relative to the pattern only)
            //
            start = 0.0;
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strStart, "stt_%s",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strStart, "stt_%s_%d",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strStart, "stt_%d_%d", i+1, j+1);
            }
            sprintf(strLine, "set %s   ", strStart);
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (k == endnote)
                    break;

                if (song->channels[i]->patterns[j]->note[k] == SONG_NOTEOFF)
                    continue;

                //
                //  Each "beat" is a 32nd note relative to ASCORE
                //  4 beats = "*"
                //
                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    int istart;

                    if (k == 0)
                        sprintf(strLine, "%s @", strLine);
                    else if ((k % 8 == 0) && k > 0)
                    {
                        //
                        //  multiple of quarter notes?
                        //
                        istart = k / 8;
                        if (istart > 0)
                        {
                            sprintf(strLine, "%s @+[%dq]", strLine, istart);
                        }
                        else
                            sprintf(strLine, "%s @+[q]", strLine);
                    }
                    else if ((k % 4 == 0) && k > 0)
                    {
                        //
                        //  multiple of eighth notes?
                        //
                        istart = k / 4;
                        if (istart > 0)
                        {
                            sprintf(strLine, "%s @+[%d*]", strLine, istart);
                        }
                        else
                            sprintf(strLine, "%s @+[*]", strLine);
                    }
                    else if ((k % 2 == 0) && k > 0)
                    {
                        //
                        //  multiple of sixteenth notes?
                        //
                        istart = k / 2;
                        if (istart > 0)
                        {
                            sprintf(strLine, "%s @+[%d!]", strLine, istart);
                        }
                        else
                            sprintf(strLine, "%s @+[!]", strLine);
                    }
                    else
                    {
                        //
                        //  multiple of 32nd notes?
                        //
                        start = (float)k / 2.0;
                        sprintf(strLine, "%s @+[%0.1f!]", strLine, start);
                    }

                } // if -- note
			} // for -- k
            fprintf(afile, "%s\n", strLine);

            //
            //  Write the duration string
            //
            dur = 0.0;
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strDur, "dur_%s",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strDur, "dur_%s_%d",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strDur, "dur_%d_%d", i+1, j+1);
            }
            sprintf(strLine, "set %s   ", strDur);
            strTmpText[0] = 0x00;
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (k == endnote)
                    break;

                if (song->channels[i]->patterns[j]->note[k] >= 0 &&
                    song->channels[i]->patterns[j]->note[k] != SONG_NOTEOFF)
                {
                    int idur, noteoff, kk;
                    //
                    //  find the next nonempty note (in the pattern)
                    //
                    dur = 0.0;

                    //
                    //  idur determines the duration of the sound
                    //
                    idur = 0;
                    noteoff = 0;
                    for (kk=k+1; kk < SONGPATLEN; kk++)
                    {
                        int asc_noteoff;

                        asc_noteoff =
                            song->channels[i]->patterns[j]->asc_noteoff;
                        idur++;
                        if (kk == endnote)
                        {
                            if (asc_noteoff == 0)
                                noteoff = 1;
                            break;
                        }
                        if (song->channels[i]->patterns[j]->note[kk] ==
                            SONG_NOTEOFF)
                        {
                            noteoff = 1;
                            break;
                        }

                        //  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
                        //  Whether we adhere to strict NOTEOFF duration.
                        //
                        //  If the asc_noteoff property is 1, then we make the
                        //  note on until a note off is found or the max
                        //  duration is exceeded.
                        //
                        //  If the asc_noteoff property is 0, then we make the
                        //  note on only until the next note (on) is found.
                        //  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
                        if (asc_noteoff == 0 &&
                                song->channels[i]->patterns[j]->note[kk] >= 0)
                        {
                            noteoff = 1;
                            break;
                        }
                    }

                    if (!noteoff)
                    {
                        idur = MAX_NOTE_IDUR;
                    }

                    //
                    //  How many idur's were counted?
                    //
                    if (idur % 32 == 0)
                    {
                        int idcnt;
                        idcnt = idur / 32;
                        if (idcnt == 1)
                            sprintf(strTmpText, "%s o", strTmpText);
                        else
                            sprintf(strTmpText, "%s %do", strTmpText, idcnt);
                    }
                    else if (idur % 16 == 0)
                    {
                        int idcnt;
                        idcnt = idur / 16;
                        if (idcnt == 1)
                            sprintf(strTmpText, "%s h", strTmpText);
                        else
                            sprintf(strTmpText, "%s %dh", strTmpText, idcnt);
                    }
                    else if (idur % 8 == 0)
                    {
                        int idcnt;
                        idcnt = idur / 8;
                        if (idcnt == 1)
                            sprintf(strTmpText, "%s q", strTmpText);
                        else
                            sprintf(strTmpText, "%s %dq", strTmpText, idcnt);
                    }
                    else if (idur % 4 == 0)
                    {
                        int idcnt;
                        idcnt = idur / 4;
                        if (idcnt == 1)
                            sprintf(strTmpText, "%s *", strTmpText);
                        else
                            sprintf(strTmpText, "%s %d*", strTmpText, idcnt);
                    }
                    else if (idur % 2 == 0)
                    {
                        int idcnt;
                        idcnt = idur / 2;
                        if (idcnt == 1)
                            sprintf(strTmpText, "%s !", strTmpText);
                        else
                            sprintf(strTmpText, "%s %d!", strTmpText, idcnt);
                    }
                    else
                    {
                        dur = (float)idur / 2.0;
                        sprintf(strTmpText, "%s %0.1f!", strTmpText, dur);
                    }

                } // if -- note
			}
            str_TruncRepeats(strTmpText);
            fprintf(afile, "%s%s\n", strLine, strTmpText);

            //
            //  Write the pitch (key) number string
            //
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strKey, "key_%s",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strKey, "key_%s_%d",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strKey, "key_%d_%d", i+1, j+1);
            }
            sprintf(strLine, "set %s   ", strKey);
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (k == endnote)
                    break;

                if (song->channels[i]->patterns[j]->note[k] == SONG_NOTEOFF)
                    continue;

                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    int midi_note, N, n, fnote;
                    midi_note = song->channels[i]->patterns[j]->note[k];
                    //
                    //  Find the pitch numbers
                    //
                    fnote=0;
                    N = 3;
                    while (N < 14 && !fnote)
                    {
                        n = 0;
                        while (n < 12 && !fnote)
                        {
                            if (midi_note == pchnote[N][n])
                            {
                                sprintf(key, "%d.%02d", N, n);
                                fnote = 1;
                            }
                            n++;
                        }
                        N++;
                    }
                    sprintf(strLine, "%s %s", strLine, key);
                }
			}
            fprintf(afile, "%s\n", strLine);

            //
            //  Write the velocity number string
            //
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strVel, "vel_%s",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strVel, "vel_%s_%d",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strVel, "vel_%d_%d", i+1, j+1);
            }
            sprintf(strLine, "set %s   ", strVel);
            strTmpText[0] = 0x00;
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (k == endnote)
                    break;

                if (song->channels[i]->patterns[j]->note[k] == SONG_NOTEOFF)
                    continue;

                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    vel = song->channels[i]->patterns[j]->vol[k];
                    sprintf(strTmpText, "%s %d", strTmpText, vel);
                    idec++;
                }
			}
            str_TruncRepeats(strTmpText);
            fprintf(afile, "%s%s\n", strLine, strTmpText);

            //
            //  Write the pattern definition line
            //
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
                sprintf(strLine, "set %s   ",
                        song->channels[i]->patterns[j]->name);
            else
            {
                if (strlen(song->channels[i]->name) > 0)
                    sprintf(strLine, "set %s_p%d   ",
                            song->channels[i]->name, j+1);
                else
                    sprintf(strLine, "set c%d_p%d   ", i+1, j+1);
            }
            sprintf(strLine, "%s     &", strLine);
            sprintf(strLine, "%s    ^%s", strLine, strInstr);
            sprintf(strLine, "%s    ^%s", strLine, strStart);
            sprintf(strLine, "%s    ^%s", strLine, strDur);
            sprintf(strLine, "%s    ^%s", strLine, strKey);
            sprintf(strLine, "%s    ^%s", strLine, strVel);
            fprintf(afile, "%s\n", strLine);


            fprintf(afile, "#\n");
            fprintf(afile, "#    ------------------------------------");
            fprintf(afile, "------------------------------------\n");
            fprintf(afile, "\n");
		}
        fprintf(afile, "\n");
        fprintf(afile, "\n");
    }

    fprintf(afile, "#\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "\n");

    //
    //  Define dummy note
    //
    fprintf(afile, "set ins_rest 0\n");
    fprintf(afile, "set dur_rest !\n");
    fprintf(afile, "set key_rest 0\n");
    fprintf(afile, "set vel_rest 0\n");
    fprintf(afile, "set rest_1   & ^ins_rest & ^dur_rest ^key_rest ^vel_rest\n");

    fprintf(afile, "#\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "\n");
    fprintf(afile, "\n");
    fprintf(afile, "\n");

    //  --------------------------------------------------------------------
    //  End  writing pattern definitions
    //  --------------------------------------------------------------------


    //  --------------------------------------------------------------------
    //
    //  Now write the SCORE section
    //
    //  --------------------------------------------------------------------
    fprintf(afile, "\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "#    SCORE Section\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "#    ------------------------------------");
    fprintf(afile, "------------------------------------\n");
    fprintf(afile, "\n");

    fprintf(afile, "TEMPO %d\n", song->tempo);

    //
    //  Get the largest song position (how many rows there will be)
    //
    max_songpos = 0;
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        int empty;

        //  only get channel if non-empty
        empty = M_TRUE;
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlank(song->channels[i]->patterns[j])==M_FALSE)
                empty = M_FALSE;
        }
        if (empty == M_TRUE)
            continue;

        for (songpos = 0; songpos < SONGLEN; ++songpos)
        {
            if (song->channels[i]->songdata[songpos])
            {
                if (songpos > max_songpos)
                    max_songpos = songpos;
            }
        }
    }

    //
    //  Set the Pause / Dummy note at the beginning
    //
    fprintf(afile, "TRACKS 1\n");
    fprintf(afile, "q             rest_1\n");
    fprintf(afile, "#\n");

    //
    //  Write initial Control Setup messages
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        int empty;

        //  only get channel if non-empty
        empty = M_TRUE;
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlankorOff(song->channels[i]->patterns[j])==M_FALSE)
                empty = M_FALSE;
        }
        if (empty == M_TRUE)
            continue;

        if (song->channels[i]->ctrl_msg)
        {
            struct SChanMidiCtrl *curr_ctrl;

            curr_ctrl = song->channels[i]->ctrl_msg;
            while (curr_ctrl)
            {
                int asc_channel;

                asc_channel = (song->channels[i]->midichannel+1) +
                              (song->channels[i]->midiport * 16);
                fprintf(afile, "CONTROL   %d   %d   %d\n",
                                curr_ctrl->control_number,
                                curr_ctrl->control_value,
                                asc_channel);

                curr_ctrl = curr_ctrl->next;
            }
        }
    }
    fprintf(afile, "#\n");

    //
    //  Find the max SongChannel (how many columns there will be)
    //
    max_chan = 0;
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        int empty;

        //  only get channel if non-empty
        empty = M_TRUE;
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlankorOff(song->channels[i]->patterns[j])==M_FALSE)
                empty = M_FALSE;
        }
        if (empty == M_TRUE)
            continue;

        max_chan++;
    }
    fprintf(afile, "TRACKS %d\n", max_chan);

    //
    //  Alignment -- make the columns lineup
    //  The score line has provided 8 characters for the timing column
    //  then 8 characters to the center of the first column (8 chars wide)
    //  A column is centered every 12 characters
    //
    iLineLength = 12*max_chan + 8;
    if (iLineLength >= MAX_ASCORE_LINE)
        iLineLength = MAX_ASCORE_LINE-1;

    //
    //  Fill with spaces -- later, overlay at fixed positions to force
    //  alignment
    //
    for (i=0; i < iLineLength; i++)
        strLine[i] = 0x20;
    strLine[iLineLength] = 0x00;


    //  -------------------------------------------------------
    //  Now enter a line of pattern names at each song position
    //  Each loop is 1 line in the SCORE
    //  -------------------------------------------------------
    for (songpos = 0; songpos <= max_songpos; ++songpos)
    {
        int maxdur;
        static char strPart[MAX_NAME_LENGTH];
        static char strName[MAX_NAME_LENGTH];

        //
        //  Loop thru the valid channels
        //  Find the pattern with the max duration
        //
        maxdur = 0xffff;
        for (i = 0; i < SONGMAXCHAN; ++i)
        {
            int empty;

            //  only get channel if non-empty
            empty = M_TRUE;
            for (j = 0; j < SONGMAXPAT; ++j)
            {
                if (spat_isBlankorOff(song->channels[i]->patterns[j])==M_FALSE)
                    empty = M_FALSE;
            }
            if (empty == M_TRUE)
                continue;


            //
            //  Valid channel
            //
            if (song->channels[i]->songdata[songpos])
            {
                int pat_dur, pat;
                //
                //  Get the pattern's duration (max number of 32nd notes)
                //
                pat = song->channels[i]->songdata[songpos];
                pat_dur = song->channels[i]->patterns[pat]->endnote;
                if (pat_dur < maxdur)
                    maxdur = pat_dur;
            }
        }

        //
        //  Clear the strLine buffer
        //
        for (i=0; i < iLineLength; i++)
            strLine[i] = 0x20;
        strLine[iLineLength] = 0x00;

        //
        //  Enter the timing column
        //
        if (maxdur % 32 == 0)
        {
            int idcnt;
            idcnt = maxdur / 32;
            if (idcnt == 1)
                sprintf(strPart, "o");
            else
                sprintf(strPart, "%do", idcnt);
        }
        else if (maxdur % 16 == 0)
        {
            int idcnt;
            idcnt = maxdur / 16;
            if (idcnt == 1)
                sprintf(strPart, "h");
            else
                sprintf(strPart, "%dh", idcnt);
        }
        else if (maxdur % 8 == 0)
        {
            int idcnt;
            idcnt = maxdur / 8;
            if (idcnt == 1)
                sprintf(strPart, "q");
            else
                sprintf(strPart, "%dq", idcnt);
        }
        else if (maxdur % 4 == 0)
        {
            int idcnt;
            idcnt = maxdur / 4;
            if (idcnt == 1)
                sprintf(strPart, "*");
            else
                sprintf(strPart, "%d*", idcnt);
        }
        else if (maxdur % 2 == 0)
        {
            int idcnt;
            idcnt = maxdur / 2;
            if (idcnt == 1)
                sprintf(strPart, "!");
            else
                sprintf(strPart, "%d!", idcnt);
        }
        else
        {
            float dur;
            dur = (float)maxdur / 2.0;
            sprintf(strPart, "%0.1f!", dur);
        }

        //
        //  Overlay this timing string into the left-hand edge of the Line
        //
        for (i=0; i < strlen(strPart); i++)
            strLine[i] = strPart[i];


        //
        //  Loop across the row -- Append the pattern names
        //
        for (i = 0; i < max_chan; ++i)
        {
            int empty, mm;

            mm = i+1;

            //
            //  Fill the strPart[] buffer with 8 spaces
            //
            for (j=0; j<NAME_LENGTH; j++)
                strPart[j] = 0x20;
            strPart[NAME_LENGTH] = 0x00;

            //
            //  check channel if non-empty
            //
            empty = M_TRUE;
            for (j = 0; j < SONGMAXPAT; ++j)
            {
                if (spat_isBlankorOff(song->channels[i]->patterns[j])==M_FALSE)
                    empty = M_FALSE;
            }
            if (empty == M_TRUE)
            {
                strPart[nCenter] = '|';
            }
            else
            {
                //
                //  Valid channel
                //
                if (song->channels[i]->songdata[songpos] > 0)
                {
                    int pat, len, nStart;

                    pat = song->channels[i]->songdata[songpos]-1;
                    if (spat_isBlankorOff(song->channels[i]->patterns[pat])==M_TRUE)
                    {
                        strPart[nCenter] = '|';
                    }
                    else
                    {
                        strcpy(strName, song->channels[i]->patterns[pat]->name);
                        len = strlen(strName);
                        len = (len > NAME_LENGTH ? NAME_LENGTH : len);
                        if (len == 0)
                        {
                            //
                            //  Create a name
                            //
                            if (strlen(song->channels[i]->name) > 0)
                                sprintf(strName, "%s_p%d",
                                        song->channels[i]->name, j+1);
                            else
                                sprintf(strName, "c%d_p%d", i+1, pat+1);
                            len = strlen(strName);
                        }
                        nStart = ((len % 2 == 0) ? nCenter - ((len/2)-1) :
                                  nCenter - (len/2) );
                        for (j=0; j < len; j++)
                            strPart[j+nStart] = strName[j];
                    }
                }
                else
                {
                    strPart[nCenter] = '|';
                }

            } // else -- not empty

            //
            //  Overlay the pattern name
            //
            for (j=0; j<8; j++)
            {
                strLine[j+(12*mm)] = strPart[j];
            }

        } // for -- i

        //
        //  Print the score line
        //
        fprintf(afile, "%s\n", strLine);
    }

    fprintf(afile, "#\n");
    fprintf(afile, "#\n");
    fprintf(afile, "#\n");
    fprintf(afile, "END\n");
    fprintf(afile, "#\n");


    // Close the file.
    fclose(afile);

    // Success!
    return M_TRUE;
}
//  function -- song_saveSongAscore
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_saveSongAscii
//
//  Description
//
//      The entry point into the main song save function.  Produces an
//      xml-like ascci file.
//
//  Arguments
//
//      song
//
//          Pointer to the Song struct instance.
//
//      filename
//
//          Pointer to string containing the file name of the new song
//          file.
//
//  Return
//
//      Returns 0 on success, nonzero on error.
//
//  -------------------------------------------------------------------------
int song_saveSongAscii(struct Song *song, const char *filename)
{
	int i, j, k;
	FILE *afile;
    static char savefilename[MAX_FILENAME_LENGTH];

    ERROR_NULL(song,M_FALSE);

    afile = fopen(filename, "w");
    if (afile == NULL)
        return M_FALSE;

    // Write version.
    fprintf(afile, "<version value=%s/>\n", VERSION);

    // Write basic song information.

	// Starting position.
    fprintf(afile, "<startpat index=%d/>\n", song->startpat+1);

    // Current tempo.
    fprintf(afile, "<tempo value=%d/>\n", song->tempo);

    // Is looping active.
    fprintf(afile, "<loopactive flag=%d/>\n", song->loopactive);

    // Loop position.
    fprintf(afile, "<looppos position=%d/>\n", song->looppos+1);

    // Loop destination.
    fprintf(afile, "<loopdest position=%d/>\n", song->loopdest+1);

    // Write each channel.
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        int empty;

        //  TBD: only save the channel if non-empty
        empty = M_TRUE;
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            if (spat_isBlank(song->channels[i]->patterns[j])==M_FALSE)
                empty = M_FALSE;
        }
        if (empty == M_TRUE)
            continue;

        if (strlen(song->channels[i]->name) > 0)
        {
            fprintf(afile, "<channel index=%d name=%s/>\n", i+1,
                    song->channels[i]->name);
        }
        else
        {
            fprintf(afile, "<channel index=%d/>\n", i+1);
        }

        // Write the channel data.
        fprintf(afile, "    <volume level=%d/>\n",
                song->channels[i]->volume+1);
        fprintf(afile, "    <devnum number=%d/>\n", song->channels[i]->devnum+1);
        fprintf(afile, "    <mute flag=%d/>\n", song->channels[i]->mute);
        fprintf(afile, "    <midichannel index=%d/>\n",
                (song->channels[i]->midichannel % 16)+1);
        fprintf(afile, "    <midiport index=%d/>\n",
                (song->channels[i]->midiport)+1);
        fprintf(afile, "    <patch index=%d/>\n",
                song->channels[i]->prognum+1);
        fprintf(afile, "    <bank index=%d/>\n",
                song->channels[i]->banknum+1);

        if (song->channels[i]->ctrl_msg)
        {
            struct SChanMidiCtrl *curr_sctrl;

            fprintf(afile, "    <control_setup>\n");
            curr_sctrl = song->channels[i]->ctrl_msg;
            while (curr_sctrl)
            {
                fprintf(afile, "        <control number=%d value=%d/>\n",
                        curr_sctrl->control_number, curr_sctrl->control_value);
                curr_sctrl = curr_sctrl->next;
            }
            fprintf(afile, "    </control_setup>\n");
        }

        fprintf(afile, "    <songdata>\n");
        for (j = 0; j < SONGLEN; ++j)
        {
            if (song->channels[i]->songdata[j])
            {
                fprintf(afile, "        <data index=%d value=%d/>\n",
                    j+1, song->channels[i]->songdata[j]);
            }
        }
        fprintf(afile, "    </songdata>\n");

        // Now write all the patterns.
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            int have_pattern;

            have_pattern = M_FALSE;
            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    have_pattern = M_TRUE;
                    break;
                }
            }
            if (have_pattern == M_FALSE)
                continue;

			// Write the pattern name: length, string.
            if (strlen(song->channels[i]->patterns[j]->name) > 0)
            {
			    fprintf(afile, "    <pattern index=%d name=%s/>\n", j+1,
                    song->channels[i]->patterns[j]->name);
            }
            else
            {
			    fprintf(afile, "    <pattern index=%d/>\n", j+1);
            }

            //
            //	Write the end note
            //
            fprintf(afile, "        <endnote number=%d/>\n",
                    song->channels[i]->patterns[j]->endnote+1);
            //
            //	Write the Ascore noteoff property
            //
            fprintf(afile, "        <asc_noteoff value=%d/>\n",
                    song->channels[i]->patterns[j]->asc_noteoff);


            for (k = 0; k < SONGPATLEN; ++k)
            {
                if (song->channels[i]->patterns[j]->note[k] >= 0)
                {
                    fprintf(afile, "        <note index=%d midinumber=%d"
                                   " slide=%d volume=%d/>\n",
                        k+1,
                        song->channels[i]->patterns[j]->note[k]+1,
                        song->channels[i]->patterns[j]->slide[k],
                        song->channels[i]->patterns[j]->vol[k]+1);
                }
			}
			fprintf(afile, "    </pattern>\n");
		}
        fprintf(afile, "</channel>\n");
    }

    // Close the file.
    fclose(afile);

    //  Save the file name
    (void)strncpy(savefilename, filename, MAX_FILENAME_LENGTH-1);
    savefilename[MAX_FILENAME_LENGTH-1] = 0x00;
    (void)strncpy(song->filename, savefilename, MAX_FILENAME_LENGTH-1);
    song->filename[MAX_FILENAME_LENGTH-1] = 0x00;

    // Success!
    return M_TRUE;
}
//  function -- song_saveSongAscii
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_loadControlSetupAscii
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadControlSetupAscii(struct Song *song, struct SongChannel *schan,
                               struct XmlToken *xmltok)
{
    int err;
    struct XmlToken *curr_xmltok;
    struct SChanMidiCtrl *sctrl=NULL, *curr_sctrl=NULL;

    ERROR_NULL(song,M_FALSE);
    ERROR_NULL(schan,M_FALSE);

    if (xmltok == NULL)
        return M_FALSE;

    curr_xmltok = xmltok;
    while (curr_xmltok)
    {
        char *plower;

        //
        //  Make the Xml tags case insensitive
        //
        plower = convert_tolower(curr_xmltok->name);

        if (strcmp(plower, "control")==0)
        {
            int control_number, control_value;

            control_number = 0;
            control_value = 0;
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "number",
                                      &control_number);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadControlSetupAscii: control"
                                   " tag has no number attribute\n");
            }

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "value",
                                      &control_value);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadControlSetupAscii: control"
                                   " tag has no value attribute\n");
            }

            //
            //  Add the control setup
            //
            if (control_number > 0 && control_value > 0)
            {
                if (sctrl == NULL)
                {
                    sctrl = curr_sctrl = Create_SChanMidiCtrl();
                }
                else
                {
                    curr_sctrl->next = Create_SChanMidiCtrl();
                    curr_sctrl = curr_sctrl->next;
                }

                if (curr_sctrl == NULL)
                    goto Error;

                //
                //  Ok
                //
                curr_sctrl->control_number = control_number;
                curr_sctrl->control_value = control_value;
                curr_sctrl->chg_ready = 1;
                schan->midisetup_ready = 1;
            }

        }
        //  if -- control

        curr_xmltok = curr_xmltok->next;
    }

    schan->ctrl_msg = sctrl;    // success
    return M_TRUE;

Error:
    Delete_SChanMidiCtrl(sctrl);
    schan->ctrl_msg = NULL;
    return M_FALSE;
}
//  function -- song_loadControlSetupAscii
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_loadSongDataAscii
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadSongDataAscii(struct Song *song, struct SongChannel *schan,
                          struct XmlToken *xmltok)
{
    int err, data_count, index, value;
    struct XmlToken *curr_xmltok;

    ERROR_NULL(song,M_FALSE);
    ERROR_NULL(schan,M_FALSE);

    if (xmltok == NULL)
        return M_FALSE;

    data_count = 0;
    curr_xmltok = xmltok;
    while (curr_xmltok)
    {
        char *plower;

        //
        //  Make the Xml tags case insensitive
        //
        plower = convert_tolower(curr_xmltok->name);

        index = 0;
        if (strcmp(plower, "data")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &index);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongDataAscii: data"
                                   " tag has no index attribute (count %d)"
                                   "\n", data_count);
            }

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "value",
                                      &value);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongDataAscii: data"
                                   " tag has no value attribute (count %d)"
                                   "\n", data_count);
            }

            if (index < 1)
                index = 1;
            if (value < 1)
                value = 1;
            schan->songdata[index-1] = value;
        }
        //  if -- note

        curr_xmltok = curr_xmltok->next;
    }

    return M_TRUE;
}
//  function -- song_loadSongDataAscii
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_loadPatternAscii
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadPatternAscii(struct Song *song, struct SongPattern *spat,
                          struct XmlToken *xmltok, int pat_index)
{
    int err, note_count;
    struct XmlToken *curr_xmltok;

    //
    //  remember: pat_index is the pattern number counting from 1,
    //  it is only used in error messages
    //

    ERROR_NULL(song,M_FALSE);
    ERROR_NULL(spat,M_FALSE);

    if (xmltok == NULL)
        return M_FALSE;

    note_count = 0;
    curr_xmltok = xmltok;
    while (curr_xmltok)
    {
        char *plower;

        //
        //  Make the Xml tags case insensitive
        //
        plower = convert_tolower(curr_xmltok->name);

        if (strcmp(plower, "endnote")==0)
        {
            int endnote=0;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "number",
                                      &endnote);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadPatternAscii: endnote"
                                   " tag has no number attribute (pattern %d)"
                                   "\n", pat_index);
            }
            if (endnote < 1)
                endnote = 1;
            spat->endnote = endnote-1;

        }
        else if (strcmp(plower, "asc_noteoff")==0)
        {
            int asc_noteoff;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "value",
                                      &asc_noteoff);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadPatternAscii: asc_endnote"
                                   " tag has no number attribute (pattern %d)"
                                   "\n", pat_index);
            }
            if (asc_noteoff < 0)
                asc_noteoff = 0;
            spat->asc_noteoff = asc_noteoff;

        }
        else if (strcmp(plower, "note")==0)
        {
            int index, midinumber, slide, volume;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &index);
            if (err)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " tag has no index attribute"
                                   " (note count %d)\n", note_count);
                note_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            if (index < 1 || index > SONGPATLEN)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " index attribute is out-of-range"
                                   " (note count %d)\n", note_count);
                note_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "midinumber",
                                      &midinumber);
            if (err)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " tag has no midinumber attribute"
                                   " (note position %d)\n", index);
                note_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            if (midinumber < 1 || midinumber > 129)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " midinumber attribute is out-of-range"
                                   " (note position %d)\n", index);
                note_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "slide",
                                      &slide);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadPatternAscii: note"
                                   " tag has no slide attribute"
                                   " (note position %d)\n", index);
                slide = 0;
            }
            if (slide < 0 || slide > 1)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " slide attribute is out-of-range"
                                   " (note position %d)\n", index);
                slide = 0;
            }
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "volume",
                                      &volume);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadPatternAscii: note"
                                   " tag has no volume attribute"
                                   " (note position %d)\n", index);
                volume = 64;
            }
            if (volume < 1 || volume > 128)
            {
                LogFile_Printf("ERROR: song_loadPatternAscii: note"
                                   " volume attribute is out-of-range"
                                   " (note position %d)\n", index);
                volume = 64;
            }
            note_count++;

            //
            //  Fill the note at this position
            //
            if (index < 1)
                index = 1;
            spat->note[index-1] = midinumber-1;
            spat->slide[index-1] = slide;
            spat->vol[index-1] = volume-1;

        }
        //  if -- note

        curr_xmltok = curr_xmltok->next;
    }

    return M_TRUE;
}
//  function -- song_loadPatternAscii
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  song_loadChannelAscii
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadChannelAscii(struct Song *song, struct SongChannel *schan,
                          struct XmlToken *xmltok, int chan_index)
{
    int err, pat_count = 0;
    int midichannel=0, midiport=0, patch=0, bank=0;
    struct XmlToken *curr_xmltok;

    ERROR_NULL(song,M_FALSE);
    ERROR_NULL(schan,M_FALSE);

    if (xmltok == NULL)
        return M_FALSE;


    curr_xmltok = xmltok;
    while (curr_xmltok)
    {
        char *plower;

        //
        //  Make the Xml tags case insensitive
        //
        plower = convert_tolower(curr_xmltok->name);

        if (strcmp(plower, "volume")==0)
        {
            int volume;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "level",
                                      &volume);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: volume"
                                   " tag has no level attribute (channel %d)"
                                   "\n", chan_index);
            }
            else
            {
                schan_setVolume(schan, volume);
                schan->midisetup_ready = 1;
            }
        }
        else if (strcmp(plower, "devnum")==0)
        {
            int devnum;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "number",
                                      &devnum);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: devnum"
                                   " tag has no number attribute (channel %d)"
                                   "\n", chan_index);
            }
            if (devnum < 1)
                devnum = 1;
            schan->devnum = devnum-1;
        }
        else if (strcmp(plower, "mute")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "flag",
                                      &schan->mute);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: mute"
                                   " tag has no flag attribute (channel %d)"
                                   "\n", chan_index);
            }
        }
        else if (strcmp(plower, "midichannel")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &midichannel);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: midichannel"
                                   " tag has no index attribute (channel %d)"
                                   "\n", chan_index);
            }
        }
        else if (strcmp(plower, "midiport")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &midiport);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: midiport"
                                   " tag has no index attribute (channel %d)"
                                   "\n", chan_index);
            }
        }
        else if (strcmp(plower, "patch")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &patch);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: patch"
                                   " tag has no index attribute (channel %d)"
                                   "\n", chan_index);
            }
            else
            {
                if (patch < 1)
                    patch = 1;
                schan->prognum = patch-1;
                schan->prchg_ready = 1;
                schan->midisetup_ready = 1;
            }
        }
        else if (strcmp(plower, "bank")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &bank);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadChannelAscii: bank"
                                   " tag has no index attribute (channel %d)"
                                   "\n", chan_index);
            }
            else
            {
                if (bank < 0)
                    bank = 0;
                schan->banknum = bank-1;
                schan->bnchg_ready = 1;
                schan->midisetup_ready = 1;
            }
        }
        else if (strcmp(plower, "control_setup")==0)
        {
            numlogtabs++;
            err = song_loadControlSetupAscii(song, schan, curr_xmltok->child);
            numlogtabs--;
        }
        else if (strcmp(plower, "songdata")==0)
        {
            //
            //  song_loadSongDataAscii()
            //
            numlogtabs++;
            err = song_loadSongDataAscii(song, schan, curr_xmltok->child);
            numlogtabs--;
        }
        else if (strcmp(plower, "pattern")==0)
        {
            int pat = 0;
            static char name[MAX_NAME_LENGTH];

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index", &pat);
            if (err)
            {
                //
                //  This should never happen, but in case someone manually
                //  edits the file....
                //
                Error_Printf("WARNING: song_loadChannelAscii: pattern tag "
                               "(count=%ld) has no index attribute\n",
                               pat_count);
                pat_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            err = Get_String_XmlAttr(curr_xmltok->attr, "name", name);

            if (pat < 1)
                pat = 1;
            if (err==0)
                strcpy(schan->patterns[pat-1]->name, name);

            //
            //  song_loadPatternAscii()
            //
            numlogtabs++;
            err = song_loadPatternAscii(song, schan->patterns[pat-1],
                                        curr_xmltok->child, pat);
            numlogtabs--;
            
            pat_count++;
        }

        curr_xmltok = curr_xmltok->next;
    }

    schan->midichannel = midichannel-1;
    if (schan->midichannel < 0)
        schan->midichannel = 0;

    schan->midiport = midiport-1;
    if (schan->midiport < 0)
        schan->midiport = 0;

    return M_TRUE;
}
//  function -- song_loadChannelAscii
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_loadSongAscii
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadSongAscii(struct Song *song, const char *filename)
{
	int err;
	FILE *afile;
    static char Line[1024];
    long charcount = 0, fposition, chan_count = 0;
    char *filebuffer = NULL;
    struct XmlToken *xmltok = NULL, *curr_xmltok;

    ERROR_NULL(song,M_FALSE);

    afile = fopen(filename, "r");
    if (afile == NULL)
    {
        Error_Printf("E260: cannot open (%s)\n", filename);
        return M_FALSE;
    }
    //
    //  Get the file size
    //
    while (feof(afile)==0)
    {
        char *pch, *plf, *pcr;

        //
        //  get the next line from the file
        //
        pch = fgets(Line, 1023, afile);
        if (pch == NULL)
            break;

        //
        //  Eliminate LFCR
        //
        pcr = strchr(Line, 0x0D);
        if (pcr)
            *pcr = 0x00;
        plf = strchr(Line, 0x0A);
        if (plf)
            *plf = 0x00;

        charcount += strlen(Line);
    }
    // while -- feof
    fclose(afile);

    //
    //  Allocate memory for the file
    //
    filebuffer = (char *)malloc( sizeof(char) * (charcount+1) );
    if (filebuffer == NULL)
    {
        Error_Printf("ERROR: song_loadSongAscii: mem alloc\n");
        return M_FALSE;
    }
    filebuffer[0] = 0x00;

    //
    //  Open the file again and read it into the file buffer
    //  without line breaks
    //
    fposition = 0;
    afile = fopen(filename, "r");
    while (feof(afile)==0)
    {
        char *pch, *plf, *pcr;

        //
        //  get the next line from the file
        //
        pch = fgets(Line, 1023, afile);
        if (pch == NULL)
            break;

        //
        //  Eliminate LFCR
        //
        pcr = strchr(Line, 0x0D);
        if (pcr)
            *pcr = 0x00;
        plf = strchr(Line, 0x0A);
        if (plf)
            *plf = 0x00;

        strcat(filebuffer, Line);
        fposition += strlen(Line);
        
    }
    fclose(afile);

    //
    //  Parse the filebuffer into an XML tree
    //
    xmltok = Read_FileBuffer(filebuffer);
    if (xmltok == NULL)
        goto Error;


    //
    // Now we can clear.
    //
    song_clear(song);

    //
    //  Load the Xml tree into the attrk data structure
    //
    curr_xmltok = xmltok;
    while (curr_xmltok)
    {
        char *plower;

        if (strlen(curr_xmltok->name) == 0)
        {
            curr_xmltok = curr_xmltok->next;
            continue;
        }

        //
        //  Make the Xml tags case insensitive
        //
        plower = convert_tolower(curr_xmltok->name);

        if (strcmp(plower, "version")==0)
        {
            //  TBD
            //  I don't care about the version of attrk.  Try to keep
            //  it independent of version
            //
        }
        else if (strcmp(plower, "startpat")==0)
        {
            int startpat;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index",
                                      &startpat);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongAscii: startpat"
                                   " tag has no index attribute\n");
            }
            if (startpat < 1)
                startpat = 1;
            song->startpat = startpat-1;
        }
        else if (strcmp(plower, "tempo")==0)
        {
            int tempo;
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "value",
                                      &tempo);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongAscii: tempo"
                                   " tag has no value attribute\n");
            }
            song_setTempo(song, tempo);
        }
        else if (strcmp(plower, "loopactive")==0)
        {
            err = Get_Integer_XmlAttr(curr_xmltok->attr, "flag",
                                      &song->loopactive);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongAscii: loopactive"
                                   " tag has no flag attribute\n");
            }
        }
        else if (strcmp(plower, "looppos")==0)
        {
            int looppos;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "position",
                                      &looppos);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongAscii: looppos"
                                   " tag has no position attribute\n");
            }
            if (looppos < 1)
                looppos = 1;
            song->looppos = looppos-1;
        }
        else if (strcmp(plower, "loopdest")==0)
        {
            int loopdest;

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "position",
                                      &loopdest);
            if (err)
            {
                LogFile_Printf("WARNING: song_loadSongAscii: loopdest"
                                   " tag has no position attribute\n");
            }
            if (loopdest < 1)
                loopdest = 1;
            song->loopdest = loopdest-1;
        }
        else if (strcmp(plower, "channel")==0)
        {
            int chan = 0;
            static char name[MAX_NAME_LENGTH];

            err = Get_Integer_XmlAttr(curr_xmltok->attr, "index", &chan);
            if (err)
            {
                //
                //  This should never happen, but in case someone manually
                //  edits the file....
                //
                Error_Printf("WARNING: song_loadSongAscii: channel tag "
                               "(count=%ld) has no index attribute\n",
                               chan_count);
                chan_count++;
                curr_xmltok = curr_xmltok->next;
                continue;
            }
            if (chan < 1)
                chan = 1;

            err = Get_String_XmlAttr(curr_xmltok->attr, "name", name);
            if (err==0)
                strcpy(song->channels[chan-1]->name, name);

            //
            //  song_loadChannelAscii()
            //
            numlogtabs++;
            err = song_loadChannelAscii(song, song->channels[chan-1],
                                        curr_xmltok->child, chan);
            numlogtabs--;
            
            chan_count++;

        }
        else
        {
            Error_Printf("ERROR: song_loadSongAscii: unknown xml tag (%s)\n",
                         plower);
        }

        curr_xmltok = curr_xmltok->next;
    }

    //
    //  Keep the filename
    //
    strncpy(song->filename, filename, (MAX_FILENAME_LENGTH-1));
    song->filename[(MAX_FILENAME_LENGTH-1)] = 0x00;

    //
    //  Execute the midi messages that affect control and program
    //
    song_doMidiSetup(song);

    //
    //  -------------------------------------------
    //  replaced with song_doMidiSetup, 5-24-2009
    //  mctrl_doCurrentMessages(p_MidiControl);
    //  -------------------------------------------

    Delete_XmlToken(xmltok);
    if (filebuffer)
        free(filebuffer);
    return M_TRUE;

Error:

    Delete_XmlToken(xmltok);
    if (filebuffer)
        free(filebuffer);
    return M_FALSE;
}
//  function -- song_loadSongAscii
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_loadSong
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_loadSong(struct Song *song, const char *filename)
{
	int version = 0;
	int i, j, k;
	int curi, val, len;
	char *tempstr;
    int maxlen, maxchan;
    gzFile fd;

    ERROR_NULL(song,-1);

    fd = gzopen( filename, "r" );
    if (fd == 0)
    {
        Error_Printf("E261: song_loadSong: cannot open file (%s)\n", filename);
        goto Error;
    }

	// Read version.
    gzread(fd, &curi, sizeof(int));
    if (curi != version)
    {
        Error_Printf("WARNING: song written in version %s\n", curi);
    }

    // Now we can clear.
    song_clear(song);

    // Starting position.
    gzread(fd, &song->startpat, sizeof(int));

    // Current tempo.
    gzread(fd, &song->tempo, sizeof(int));

    // Is looping active.
    gzread(fd, &song->loopactive, sizeof(int));

    // Loop position.
    gzread(fd, &song->looppos, sizeof(int));

    // Loop destination.
    gzread(fd, &song->loopdest, sizeof(int));

    // pattern length
    gzread(fd, &maxlen, sizeof(int));
    maxlen = (maxlen <= SONGPATLEN ? maxlen : SONGPATLEN);
    // chan length
    gzread(fd, &maxchan, sizeof(int));
    maxchan = (maxchan <= SONGMAXCHAN ? maxchan : SONGMAXCHAN);



	// Read each channel.
    for (i = 0; i < maxchan; ++i )
    {
        // Read the channel data.
        gzread(fd, &(song->channels[i]->midichannel), sizeof(int));
        gzread(fd, &(song->channels[i]->volume), sizeof(int));
        gzread(fd, &(song->channels[i]->devnum), sizeof(int));
        gzread(fd, &(song->channels[i]->mute), sizeof(int));

        // Read the channel name: length, string.
        gzread(fd, &len, sizeof(int));
        tempstr = (char *)malloc( sizeof(char) * len );
        gzread(fd, tempstr, len);
		schan_setName(song->channels[i], tempstr);

        // Read song pattern indexes.
        for (;;)
        {
            gzread(fd, &curi, sizeof(int));
            if(curi == INDEX_DONE)
                break;

            gzread(fd, &val, sizeof(int));
			song->channels[i]->songdata[curi] = val;
        }

        // Now read all the patterns.
        for (j = 0; j < SONGMAXPAT; ++j)
        {
            gzread(fd, &curi, sizeof(int));
            if (curi == PAT_BLANK)
                continue;

            // Read the channel name: length, string.
            gzread(fd, &len, sizeof(int));
            tempstr = (char *)malloc( sizeof(char) * (len+1));
            gzread(fd, tempstr, len);
			spat_setName(song->channels[i]->patterns[j], tempstr);
			free(tempstr);

			//	Read endnote
            gzread(fd, &song->channels[i]->patterns[j]->endnote, sizeof(int));
            for(k = 0; k < maxlen; ++k)
            {
                gzread(fd, &(song->channels[i]->patterns[j]->note[k]), sizeof(int));
                gzread(fd, &(song->channels[i]->patterns[j]->slide[k]), sizeof(int));
				gzread(fd, &(song->channels[i]->patterns[j]->vol[k]), sizeof(int));
            }
        }
    }

    // Close the file.
    gzclose(fd);

    // Success!
    return 0;

Error:
    return -1;
}
//  function -- song_loadSong
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_setLoopPosition
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_setLoopPosition(struct Song *song, int newpos)
{
    ERROR_NULL_VOID(song);

    if (newpos < 0 || newpos >= SONGLEN)
    {
        Error_Printf("E262: song_setLoopPosition: position out-of-range\n");
        return;
    }
	song->looppos = newpos;
}
//  function -- song_setLoopPosition
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_setLoopDestination
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_setLoopDestination(struct Song *song, int newdest)
{
    ERROR_NULL_VOID(song);

    if (newdest < 0 || newdest >= SONGLEN)
    {
        Error_Printf("E263: song_setLoopDestination: dest out-of-range\n");
        return;
    }
	song->loopdest = newdest;
}
//  function -- song_setLoopDestination
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_start
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_start(struct Song *song)
{
    ERROR_NULL_VOID(song);
    song->playing = M_TRUE;
}
//  function -- song_start
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_stop
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_stop(struct Song *song)
{
    ERROR_NULL_VOID(song);
    song->playing = M_FALSE;
}
//  function -- song_stop
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_rewind
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_rewind(struct Song *song)
{
    ERROR_NULL_VOID(song);

    song->curpat = song->startpat;
    song->curbeat = 0;
}
//  function -- song_rewind
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_isPlaying
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_isPlaying(struct Song *song)
{
    ERROR_NULL(song,M_FALSE);
    return song->playing;
}
//  function -- song_isPlaying
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getCurPat
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getCurPat(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->curpat;
}
//  function -- song_getCurPat
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getCurBeat
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getCurBeat(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->curbeat;
}
//  function -- song_getCurBeat
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getStartPos
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getStartPos(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->startpat;
}
//  function -- song_getStartPos
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_setStartPos
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_setStartPos(struct Song *song, int newstartpat)
{
    ERROR_NULL_VOID(song);
    song->startpat = newstartpat;
}
//  function -- song_setStartPos
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongChannel *song_getChannel(struct Song *song, int chan)
{
    ERROR_NULL(song,NULL);
    return song->channels[chan];
}
//  function -- song_getChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getTempo
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getTempo(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->tempo;
}
//  function -- song_getTempo
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getUsecpb
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getUsecpb(struct Song *song)
{
    ERROR_NULL(song,-1);
    return song->usecpb;
}
//  function -- song_getUsecpb
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_isLoopActive
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_isLoopActive(struct Song *song)
{
    ERROR_NULL(song,M_FALSE);
    return song->loopactive;
}
//  function -- song_isLoopActive
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_toggleLoop
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void song_toggleLoop(struct Song *song)
{
    ERROR_NULL_VOID(song);
    song->loopactive = !song->loopactive;
}
//  function -- song_toggleLoop
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_getLoopPosition
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getLoopPosition(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->looppos;
}
//  function -- song_getLoopPosition
//  -------------------------------------------------------------------------

//  -------------------------------------------------------------------------
//  Function    --  song_getLoopDestination
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int song_getLoopDestination(struct Song *song)
{
    ERROR_NULL(song,0);
    return song->loopdest;
}
//  function -- song_getLoopDestination
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_armMidiSetupMessages
//
//  Description
//
//      Prep all SongChannel's that have valid notes for a call to
//      mctrl_doSetupMessages().  That is, set all their "sticky bit"
//      fields that signify that a change has been made to the parameter.
//      Then a call to mctrl_doCurrentMessages will result in the midi
//      parameters being sent to the instruments.
//
//  Arguments
//
//      song
//
//          The pointer to the current song struct.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void song_armMidiSetupMessages(struct Song *song)
{
    int i;

    ERROR_NULL_VOID(song);

    //
    //  Create each of the SongChannel structs
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        struct SongChannel *schan;
        int channel_active;

        channel_active = 0;
        schan = song->channels[i];
        if (schan)
        {
            int j;

            for (j = 0; j < SONGMAXPAT; ++j)
            {
                struct SongPattern *spat;

                spat = schan->patterns[j];
                if (spat)
                {
                    int k;

                    for (k=0; k < SONGPATLEN; ++k)
                    {
                        if (spat->note[k] > 0)
                        {
                            channel_active = 1;
                            break;
                        }
                    } // for -- k
                }

                if (channel_active)
                    break;

            } // for -- j

            if (channel_active)
            {
                //
                //  Turn on the midi change flags for this channel
                //
                schan->prchg_ready = 1;
                schan->bnchg_ready = 1;
                schan->vlchg_ready = 1;

                //
                //  set chg_ready field in linked list of control
                //  messages
                //
                if (schan->ctrl_msg)
                {
                    struct SChanMidiCtrl *curr_sctrl;

                    curr_sctrl = schan->ctrl_msg;
                    while (curr_sctrl)
                    {
                        curr_sctrl->chg_ready = 1;
                        curr_sctrl = curr_sctrl->next;
                    }
                }

                schan->midisetup_ready = 1; // OR of all the "change ready"
            }
        }
    } // for -- i
}
//  function -- song_armMidiSetupMessages
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_doMidiSetup
//
//  Description
//
//      TBD -- use this when song is loaded instead of
//      mctrl_doCurrentMessages.  But make sure that song_loadSongAscii
//      sets all the "midisetup_ready" flags.
//
//  Arguments
//
//      song
//
//          The pointer to the current song struct.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void song_doMidiSetup(struct Song *song)
{
    struct SongChannel *schan;
    int i;

    ERROR_NULL_VOID(song);

    //
    //  Send the MIDI messages if that channel has valid notes
    //
    for (i = 0; i < SONGMAXCHAN; ++i)
    {
        schan = song->channels[i];
        if (schan->midisetup_ready)
        {
            //
            //  Send all midi setup messages
            //  (mctrl_doSetupMessages is equivalent to
            //   mctrl_doCurrentMessages)
            //
            mctrl_doSetupMessages(p_MidiControl, schan);
            schan->midisetup_ready = 0;
        }
    }
}
//  function -- song_doMidiSetup
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  song_refreshMidiSetup
//
//  Description
//
//      Re-send all MIDI setup messages to the instruments.  This includes
//      the program (patch) and bank, and the volume.  The message is only
//      re-sent for active channels--channels with at least one note.
//      (TBD -- send MIDI setup control messages (on ctrl_msg linked list))
//      This function was designed to be called at an extended control
//      command (<colon> <letter command>)
//          :m r
//
//  Arguments
//
//      song
//
//          The pointer to the current song struct.
//
//  Return
//
//      Void
//
//  -------------------------------------------------------------------------
void song_refreshMidiSetup(struct Song *song)
{
    ERROR_NULL_VOID(song);

    //
    //  Set the "change ready" flags on all SongChannels that have
    //  at least one valid note.
    //
    song_armMidiSetupMessages(song);

    //
    //  Send the MIDI messages if that channel has valid notes
    //
    song_doMidiSetup(song);

}
//  function -- song_refreshMidiSetup
//  -------------------------------------------------------------------------



