/**
 * C-code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <slang.h>
#include <defs.h>
#include <errorprintf.h>
#include <xmltools.h>
#include <song.h>
#include <screen.h>
#include <songbar.h>


extern struct Song *p_Song;


//  -------------------------------------------------------------------------
//  Function    --  Init_SongBar
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void Init_SongBar(struct SongBar *sbar, int initchan, int startrow,
                  int startcol)
{
    ERROR_NULL_VOID(sbar);

	sbar->chan = initchan;
	sbar->row = startrow;
	sbar->col = startcol;
	sbar->cursor = -1;
}
//  function -- Init_SongBar
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Create_SongBar
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct SongBar *Create_SongBar(int initchan, int startrow, int startcol)
{
    struct SongBar *sbar;

    sbar = (struct SongBar *)malloc(sizeof(struct SongBar));
    if (sbar == NULL)
    {
        Error_Printf("E300: Create_SongBar: mem alloc\n");
        return NULL;
    }

    Init_SongBar(sbar, initchan, startrow, startcol);
    return sbar;
}
//  function -- Create_SongBar
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setCursor
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int sbar_setCursor(struct SongBar *sbar, int newpos)
{
	int scrsize;
    int new_scrolltop;  // the new scrolltop
    int curr_scrolltop, curr_scrollbottom;
    int maxindex = SONGLEN-1;
    int maxscrolltop;

    ERROR_NULL(sbar,0);

    sbar->cursor = newpos;
    scrsize = SLtt_Screen_Rows - sbar->row - HEADERSIZE - BOTTOMSIZE;
    curr_scrolltop = sbar->scrolltop;   // top-most visible index
    curr_scrollbottom = curr_scrolltop + scrsize - 1; // bottom most index
    maxscrolltop = maxindex-scrsize+1;

    if (scrsize < SONGLEN)
    {
        if (newpos < 0)
        {
            return 0;
        }
        else if (newpos > maxindex)
        {
            return maxscrolltop;
        }
        else
        {
            if (newpos > curr_scrollbottom)
            {
                new_scrolltop = newpos - scrsize + 1;
                if (new_scrolltop < 0)
                    new_scrolltop = 0;
                if (new_scrolltop > maxscrolltop)
                    new_scrolltop =  maxscrolltop;
            }
            else if (newpos < curr_scrolltop)
            {
                new_scrolltop = newpos;
                if (new_scrolltop < 0)
                    new_scrolltop = 0;
                if (new_scrolltop > maxscrolltop)
                    new_scrolltop =  maxscrolltop;
            }
            else
                new_scrolltop = curr_scrolltop;

            return new_scrolltop;
		}
    }
    else
    {
        return 0;
    }
}
//  function -- sbar_setCursor
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  sbar_refresh
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_refresh(struct SongBar *sbar)
{
    int i;

    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E301: sbar_refresh: NULL SongChannel\n");
        return;
    }

    SLsmg_gotorc( sbar->row, sbar->col );
    SLsmg_printf( "%-8s", schan_getName(schan));

    SLsmg_gotorc( sbar->row + 1, sbar->col );
    SLsmg_printf( "%02x     ", sbar->chan );

    SLsmg_gotorc( sbar->row + 2, sbar->col );
    SLsmg_printf( "Vol: %02d ", schan_getVolume(schan));

    SLsmg_gotorc( sbar->row + 3, sbar->col );
    SLsmg_printf( "Ch : %02d ", schan_getMidiChannel(schan) + 1 );

    SLsmg_gotorc( sbar->row + 4, sbar->col );
    if (sbar->cursor != -1)
    {
        struct SongPattern *pat = schan_getSongPattern(schan, sbar->cursor);
        if (pat)
        {
            SLsmg_printf( "%-8s", spat_getName(pat));
        }
        else
        {
            SLsmg_printf( "        " );
        }
    }
    else
    {
        SLsmg_printf( "        " );
    }

    SLsmg_gotorc(sbar->row + 5, sbar->col);
    SLsmg_printf( "========" );

    for(i = 0; i < SONGLEN; i++ ) {
        sbar_printLine(sbar, i);
    }
}
//  function -- sbar_refresh
//  -------------------------------------------------------------------------



//  -------------------------------------------------------------------------
//  Function    --  sbar_editPattern
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_editPattern(struct SongBar *sbar,  int mod)
{
    struct SongChannel *schan;
	struct SongPattern *pat;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E302: sbar_editPattern: NULL SongChannel\n");
        return;
    }


    pat = schan_getSongPattern(schan, sbar->cursor);
    if (pat)
    {
        int val = spat_getIndex(pat);
        schan_setSongPattern(schan, sbar->cursor, val + mod);
    }
    else if (mod == 1)
    {
        schan_setSongPattern(schan, sbar->cursor, 1);
    }
    sbar_refresh(sbar);
}
//  function -- sbar_editPattern
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_printLine
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_printLine(struct SongBar *sbar, int i)
{
    int printrow;
    int scrsize;
    struct SongChannel *schan;
	struct SongPattern *pat;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E303: sbar_printLine: NULL SongChannel\n");
        return;
    }


    scrsize = SLtt_Screen_Rows - sbar->row - HEADERSIZE - BOTTOMSIZE;
    if (i < sbar->scrolltop || i >= (sbar->scrolltop + scrsize))
        return;
    printrow = sbar->row + HEADERSIZE + i - sbar->scrolltop;

    SLsmg_gotorc(printrow, sbar->col);
    if (i == sbar->cursor && i % 4 == 0)
        SLsmg_set_color(HILIGHTCURSOR_COLOR);
    else if (i == sbar->cursor)
        SLsmg_set_color(CURSOR_COLOR);
    else if (i % 4 == 0)
        SLsmg_set_color(HILIGHT_COLOR);
    else if (schan_isMuted(schan))
        SLsmg_set_color(MUTEDCHANNEL_COLOR);

    pat = schan_getSongPattern(schan, i);
    if (pat)
    {
        SLsmg_printf( "   %02x   ", spat_getIndex(pat) );
    }
    else
    {
        SLsmg_printf( "   --   " );
	}
    SLsmg_set_color(DEFAULT_COLOR);
}
//  function -- sbar_printLine
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_patternSet
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int sbar_patternSet(struct SongBar *sbar)
{
    struct SongChannel *schan;

    ERROR_NULL(sbar,0);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E304: sbar_patternSet: NULL SongChannel\n");
        return 0;
    }

    if (sbar->cursor == -1)
        return M_FALSE;
	return ( schan_getSongPattern(schan, sbar->cursor) != 0);
}
//  function -- sbar_patternSet
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setPatternName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_setPatternName(struct SongBar *sbar, const char *newname)
{
    struct SongChannel *schan;
	struct SongPattern *pat;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E305: sbar_setPatternName: NULL SongChannel\n");
        return;
    }

	pat = schan_getSongPattern(schan, sbar->cursor);
    if (pat == NULL)
        return;
	spat_setName(pat, newname);
	sbar_refresh(sbar);
}
//  function -- sbar_setPatternName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_toggleChannelMute
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_toggleChannelMute(struct SongBar *sbar)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E306: sbar_toggleChannelMute: NULL SongChannel\n");
        return;
    }

    schan_toggleMute(schan);
    sbar_refresh(sbar);
}
//  function -- sbar_toggleChannelMute
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_editMidiChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_editMidiChannel(struct SongBar *sbar, int mod)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E307: sbar_editMidiChannel: NULL SongChannel\n");
        return;
    }

    schan_setMidiChannel(schan, schan_getMidiChannel(schan)+mod);
    sbar_refresh(sbar);
}
//  function -- sbar_editMidiChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_editMidiPort
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_editMidiPort(struct SongBar *sbar, int port)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E308: sbar_editMidiPort: NULL SongChannel\n");
        return;
    }

    schan_setMidiPort(schan, port);
    sbar_refresh(sbar);
}
//  function -- sbar_editMidiPort
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_editChannelVolume
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_editChannelVolume(struct SongBar *sbar, int mod)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    //
    //  Get the current "song channel" struct pointer
    //
    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E309: sbar_editChannelVolume: NULL SongChannel\n");
        return;
    }

    schan_setVolume(schan, schan_getVolume(schan)+mod);
    sbar_refresh(sbar);
}
//  function -- sbar_editChannelVolume
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_getIndex
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int sbar_getIndex(struct SongBar *sbar)
{
    ERROR_NULL(sbar,0);
    return sbar->chan;
}
//  function -- sbar_getIndex
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setChannel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_setChannel(struct SongBar *sbar, int chan)
{
    ERROR_NULL_VOID(sbar);

    sbar->chan = chan;
    sbar_refresh(sbar);
}
//  function -- sbar_setChannel
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setScrollTop
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_setScrollTop(struct SongBar *sbar, int newpos)
{
    ERROR_NULL_VOID(sbar);

    sbar->scrolltop = newpos;
    sbar_refresh(sbar);
}
//  function -- sbar_setScrollTop
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setChannelName
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_setChannelName(struct SongBar *sbar, const char *newname)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);
    ERROR_NULL_VOID(newname);

    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E310: sbar_setChannelName: NULL SongChannel\n");
        return;
    }

    schan_setName(schan, newname);
    sbar_refresh(sbar);
}
//  function -- pbar_setChannelName
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  sbar_setChannelPatch
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void sbar_setChannelPatch(struct SongBar *sbar, int bank, int program)
{
    struct SongChannel *schan;

    ERROR_NULL_VOID(sbar);

    schan = song_getChannel(p_Song, sbar->chan);
    if (schan == NULL)
    {
        Error_Printf("E311: sbar_setChannelPatch: NULL SongChannel\n");
        return;
    }

    schan_setBankNumber(schan, bank);
    schan_setProgramNumber(schan, program);
}
//  function -- sbar_setChannelPatch
//  -------------------------------------------------------------------------



