/**
 * C-code modifications Copyright (C) 2009 Alfred Steffens Jr.
 * based on TekTracker, Copyright (C) 2001 Billy Biggs and Andrew Lewycky.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <assert.h>
#include <pthread.h>
#include <signal.h>
#include <stdlib.h>
#include <stdarg.h>
#include <defs.h>
#include <logprintf.h>
#include <errorprintf.h>
#include <thread.h>

int thread_main();

extern struct MidiControl *p_MidiControl;
int g_ThreadOk = 0;

//  -------------------------------------------------------------------------
//  Function    --  Create_Thread
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
struct Thread *Create_Thread(void)
{
    struct Thread *pth;

    pth = (struct Thread *)malloc( sizeof(struct Thread) );
    if (pth == NULL)
    {
        Error_Printf("E350: Create_Thread: mem alloc\n");
        return NULL;
    }

    pth->thread_handle = 0;
    pth->running = M_FALSE;
    pth->next = NULL;

    return pth;
}
//  function -- Create_Thread
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  Delete_Thread
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void Delete_Thread(struct Thread *pth)
{
    struct Thread *pP, *pN;

    if (pth == NULL)
        return;

    pP = pth;
    while (pP)
    {
        pN = pP;
        pP = pP->next;
        thread_stop(pN);
        free(pN);     
    }
}
//  function -- Delete_Thread
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  thread_thunk
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void *thread_thunk( void *pthis )
{
    // Make sure we don't get killed early.
    pthread_setcanceltype( PTHREAD_CANCEL_DEFERRED, 0 );

    if (thread_main(p_MidiControl))
    {
        Error_Printf("E351: thread_main returned ERROR\n");
        return NULL;
    }
    return NULL;
}
//  function -- thread_thunk
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  thread_run
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
int thread_run(struct Thread *pth)
{
    ERROR_NULL(pth,M_FALSE);

	assert(!pth->running);
    g_ThreadOk = 1;
	if (pthread_create( &pth->thread_handle, NULL, thread_thunk, NULL) != 0)
    {
        Error_Printf("E352: thread_run: error received from pthread_create\n");
		return M_FALSE;
	}
	pth->running = M_TRUE;
	return M_TRUE;
}
//  function -- thread_run
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  thread_stop
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void thread_stop(struct Thread *pth)
{
    void *pthval;
    int err;

    if (pth == NULL)
    {
        g_ThreadOk = 0;
        return;
    }

    if (pth->running == M_FALSE)
    {
        g_ThreadOk = 0;
        return;
    }

    if (g_ThreadOk == 0)
        return;

    //
    //  global flag -- gentle attempt to stop thread
    //
    g_ThreadOk = 0;

    LogFile_Printf("I010: thread_stop:\n");
    err = pthread_cancel(pth->thread_handle);
    if (err)
    {
        Error_Printf("E353: error num (%d) from pthread_cancel\n", err);
        return;
    }

    LogFile_Printf("I011: thread_stop: success on pthread_cancel\n");
    err = pthread_join(pth->thread_handle, &pthval);
    if (err)
    {
        Error_Printf("E354: error num (%d) from pthread_join\n", err);
        return;
    }

    LogFile_Printf("I012: thread_stop: success on pthread_join\n");
    pth->running = M_FALSE;
}
//  function -- thread_stop
//  -------------------------------------------------------------------------


//  -------------------------------------------------------------------------
//  Function    --  thread_testcancel
//
//  Description
//
//  Arguments
//
//  Return
//
//  -------------------------------------------------------------------------
void thread_testcancel(void)
{
    pthread_testcancel();
}
//  function -- thread_testcancel
//  -------------------------------------------------------------------------


